/*
 * @(#)DatenVew.java   01.03.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.Data;

import de.inovat.buv.gtm.datvew.Activator;
import de.inovat.buv.gtm.datvew.tls.De;
import de.inovat.buv.gtm.datvew.tls.Eak;
import de.inovat.buv.gtm.datvew.tls.Geraet;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.Status;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Datenverwaltung ist als Singleton realisiert.
 * Initialisiert die Daten und erlaubt den Zugriff auf Daten.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class DatenVew {

    /** Comparator fr ISystemObjekt. Sortiert nach Pid oder Id. */
    public static Comparator<ISystemObjekt> COMPARATOR_SO_PID = new Comparator<ISystemObjekt>() {
        @Override
        public int compare(ISystemObjekt o1, ISystemObjekt o2) {
            return o1.getSystemObjekt().getPidOrId().compareTo(o2.getSystemObjekt().getPidOrId());
        }
    };

    /** Comparator fr ISystemObjekt. Sortiert nach NAme oder Pid oder Id. */
    public static Comparator<ISystemObjekt> COMPARATOR_SO_NAME = new Comparator<ISystemObjekt>() {
        @Override
        public int compare(ISystemObjekt o1, ISystemObjekt o2) {
            return o1.getSystemObjekt().getNameOrPidOrId().compareTo(o2.getSystemObjekt().getNameOrPidOrId());
        }
    };

    /** Einzige Instanz der Klasse. */
    private static DatenVew _instanz = new DatenVew();

    /** Die Datenverteiler-Konfiguration. */
    private static DataModel _davKonf;

    //~ FELDER ================================================================

    /**
     * Stellt fest, ob die Initialisierung der Gerte abgeschlossen ist (s.{@link #DatenVew()}).
     *
     * WICHTIG:
     *
     * Die get-Methoden fr folgende Variablen MSSEN auf das
     * Ende der Initialisierung warten:
     * {@link #_listeTlsRootGeraete}
     * {@link #_mapTlsGeraete}
     * {@link #_mapDe}
     */
    private boolean _istInitialisierungAbgeschlossen = false;

    /** Nach PID sortierte Liste der Konfigurationsbereichsobjekte. */
    private List<DaVKonfigurationsbereichsObjekt> _listeKB;

    /** Nach PID sortierte Liste aller Straen. */
    private List<Strasse> _listeStrassen;

    /** Nach PID sortierte Liste aller alle Root-Gerte. */
    private List<Geraet> _listeTlsRootGeraete;

    /** Nach PID sortierte Liste der DaV-ObjektTypen. */
    private List<DaVObjektTyp> _listeTyp;

    /** Nach PID sortierte Liste der ZaehlStellen. */
    private List<ZaehlStelle> _listeZaehlStellen;

    /**
     * Map aller De.
     * Key: SystemObject-PID oder ID.
     */
    private Map<String, De> _mapDe;

    /**
     * Map mit allen Fahrbahnen.
     * ber das Map wird Parent-Fahrbahnen zu DE ermittelt.
     *
     * Key: PID oder ID der FahrstreifenQuelle
     */
    private Map<String, Fahrbahnen> _mapFahrbahnen;

    /**
     * Map alle Konfigurationsbereichsobjekt mit Kindern.
     * Key: PID oder ID von DaVKonfigurationsbereichsObjekt,
     * Value: Liste der Kinder.
     */
    private Map<String, List<DaVObjektTyp>> _mapKBKinder;

    /**
     * Map aller Straen.
     * Key: SystemObject-PID oder ID.
     */
    private Map<String, Strasse> _mapStrassen;

    /**
     * Map PID oder ID der Strae als Schlssel,
     * Wert ist eine Liste aller zugehrigen nach PID-sortierten Langzeitzhlstellen.
     */
    private Map<String, List<ZaehlStelle>> _mapStrasseZaehlStellen;

    /** Map aller Gerte. */
    private Map<String, Geraet> _mapTlsGeraete;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse DatenVew.
     */
    private DatenVew() {

        //
        _davKonf = DavDatenVew.getInstanz().getDav().getDataModel();

        //
        initStrassen();
        initListeZaehlStellen();
        initMapFahrbahnen();
        initKbUndTyp();

        // -------------------------------------------------------------------
        // WICHTIG
        // Die Initialisierung der Gerte luft langsam ===>
        // wird in JOB ausgelagert.
        // ===>
        // Die get-Methoden fr folgende Variablen MSSEN auf das
        // Ende der Initialisierung warten
        // _listeTlsRootGeraete
        // _mapTlsGeraete
        // _mapDe.
        Job job = new Job("Initialisiere alle TLS-Gerte...") {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                initGeraete();

                //
                _istInitialisierungAbgeschlossen = true;

                return Status.OK_STATUS;
            }
        };

        job.schedule();
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle De mit der angegeben SystemObject-PID.
     *
     * @param pid SystemObject-PID oder ID.
     *
     * @return De mit der angegeben SystemObject-PID oder null.
     */
    public De ermittleDe(String pid) {
        pauseBisEndeDerInitialisierung();

        return _mapDe.get(pid);
    }

    /**
     * Ermittle Fahrbahnen zu angegebener FahrstreifenQuelle-PID.
     *
     * @param fsQuellePid FahrstreifenQuelle-PID.
     *
     * @return Fahrbahnen zu angegebener FahrstreifenQuelle-PID oder null.
     */
    public Fahrbahnen ermittleFahrbahnen(String fsQuellePid) {
        return _mapFahrbahnen.get(fsQuellePid);
    }

    /**
     * Ermittle alle Kinder zum angegebenen Konfigurationsbereichsobjekt.
     *
     * @param kbPid PID oder ID von Konfigurationsbereichsobjekt.
     *
     * @return Alle Kinder zum angegebenen Konfigurationsbereichsobjekt oder leere Liste.
     */
    public List<DaVObjektTyp> ermittleKBKinder(String kbPid) {
        List<DaVObjektTyp> liste = _mapKBKinder.get(kbPid);

        if (liste == null) {
            liste = new ArrayList<DaVObjektTyp>();
        }

        return liste;
    }

    /**
     * Ermittle die Liste der zu Strae zugehrigen ZaehlStellen.
     *
     * @param strassenPid PID oder ID der Strae.
     *
     * @return Liste der zu Strae zugehrigen ZaehlStellen oder leere Liste.
     */
    public List<ZaehlStelle> ermittleListeZaehlStellen(String strassenPid) {
        List<ZaehlStelle> liste = _mapStrasseZaehlStellen.get(strassenPid);

        if (liste == null) {
            liste = new ArrayList<ZaehlStelle>();
        }

        return liste;
    }

    /**
     * Ermittle die Strae mit der angegeben SystemObject-PID.
     *
     * @param pid SystemObject-PID oder ID.
     *
     * @return Strae mit der angegeben SystemObject-PID oder null.
     */
    public Strasse ermittleStrasse(String pid) {
        return _mapStrassen.get(pid);
    }

    /**
     * Ermittle das Tls-Gert mit dem angegeneben Pid.
     *
     * @param pid SystemObject-PID oder ID.
     *
     * @return Tls-Gert mit dem angegeneben Pid oder null.
     */
    public Geraet ermittleTlsGeraet(String pid) {
        pauseBisEndeDerInitialisierung();

        return _mapTlsGeraete.get(pid);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittelt einzige Instanz der Klasse DatenVew.
     *
     * @return Einzige Instanz der Klasse DatenVew.
     */
    public static DatenVew getInstanz() {
        return _instanz;
    }

    /**
     *     Getter fr Eigenschaft 'listeKB'.
     *
     *     @return Wert der Eigenschaft 'listeKB'.
     *
     *     @see #_listeKB
     */
    public List<DaVKonfigurationsbereichsObjekt> getListeKB() {
        return _listeKB;
    }

    /**
     * Getter fr Eigenschaft 'listeStrassen'.
     *
     * @return Wert der Eigenschaft 'listeStrassen'.
     *
     * @see #_listeStrassen
     */
    public List<Strasse> getListeStrassen() {
        return _listeStrassen;
    }

    /**
     * Getter fr Eigenschaft 'listeTlsRootGeraete'.
     *
     * @return Wert der Eigenschaft 'listeTlsRootGeraete'.
     *
     * @see #_listeTlsRootGeraete
     */
    public List<Geraet> getListeTlsRootGeraete() {
        pauseBisEndeDerInitialisierung();

        return _listeTlsRootGeraete;
    }

    /**
     *     Getter fr Eigenschaft 'listeTyp'.
     *
     *     @return Wert der Eigenschaft 'listeTyp'.
     *
     *     @see #_listeTyp
     */
    public List<DaVObjektTyp> getListeTyp() {
        return _listeTyp;
    }

    /**
     * Getter fr Eigenschaft 'listeZaehlStellen'.
     *
     * @return Wert der Eigenschaft 'listeZaehlStellen'.
     *
     * @see #_listeZaehlStellen
     */
    public List<ZaehlStelle> getListeZaehlStellen() {
        return _listeZaehlStellen;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere alle Gerte.
     */
    private void initGeraete() {

        // -------------------------------------------------------------
        // Initialisiere alle Root-Gerte:
        _listeTlsRootGeraete = new ArrayList<Geraet>();

        //
        List<SystemObject> alleGeraeteSo;

        // Alle Gerte:
        try {
            alleGeraeteSo = _davKonf.getType("typ.gert").getElements();
        }
        catch (Exception e) {
            alleGeraeteSo = new ArrayList<SystemObject>();
        }

        // Ermittle alle existierende Gerte:
        Map<String, SystemObject> mapAlleGeraete = new HashMap<String, SystemObject>();

        for (SystemObject geraetSo : alleGeraeteSo) {
            mapAlleGeraete.put(geraetSo.getPid(), geraetSo);
        }

        // Ermittle Root-Knoten:
        AttributeGroup     atgApk = _davKonf.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
        List<SystemObject> allApkSo;

        try {
            allApkSo = _davKonf.getType("typ.anschlussPunktKommunikationsPartner").getElements();
        }
        catch (Exception e) {
            allApkSo = new ArrayList<SystemObject>();
        }

        Data[] arConfDaten = _davKonf.getConfigurationData(allApkSo, atgApk);

        for (Data data : arConfDaten) {
            String s = data.getReferenceValue("KommunikationsPartner").getSystemObjectPid();

            mapAlleGeraete.remove(s);
        }

        for (SystemObject rootKnoten : mapAlleGeraete.values()) {
            Geraet geraet = new Geraet(rootKnoten, null);

            _listeTlsRootGeraete.add(geraet);
        }

        Collections.sort(_listeTlsRootGeraete, COMPARATOR_SO_NAME);

        // -------------------------------------------------------------
        // Initialisiere das Map aller Gerte und Map aller De:
        _mapTlsGeraete = new HashMap<String, Geraet>();
        _mapDe         = new HashMap<String, De>();
        initMapTlsGeraete(_listeTlsRootGeraete.toArray());
    }

    /**
     * Initialisiere alle Konfigurationsbereichsobjekte und DaV-Type.
     */
    private void initKbUndTyp() {

        // --------------------------------------------------------
        _listeKB = new ArrayList<DaVKonfigurationsbereichsObjekt>();

        List<SystemObject> listeKB;

        try {
            listeKB = _davKonf.getType("typ.konfigurationsBereich").getElements();
        }
        catch (Exception e) {
            listeKB = new ArrayList<SystemObject>();
        }

        for (SystemObject soKB : listeKB) {
            ConfigurationArea               conf = soKB.getConfigurationArea();
            DaVKonfigurationsbereichsObjekt kb   = new DaVKonfigurationsbereichsObjekt(conf);

            _listeKB.add(kb);
        }

        //
        Collections.sort(_listeKB, COMPARATOR_SO_PID);

        // --------------------------------------------------------
        _mapKBKinder = new HashMap<String, List<DaVObjektTyp>>();

        for (DaVKonfigurationsbereichsObjekt kb : _listeKB) {
            List<DaVObjektTyp>       listeKinder           = new ArrayList<DaVObjektTyp>();
            Collection<SystemObject> listeAllerObjekteImKB = ((ConfigurationArea) kb.getSystemObjekt()).getCurrentObjects();
            Set<SystemObject>        setAlleTyp            = new HashSet<SystemObject>();

            for (SystemObject so : listeAllerObjekteImKB) {
                SystemObjectType typ = so.getType();

                if (setAlleTyp.add(typ)) {
                    DaVObjektTyp davTyp = new DaVObjektTyp(kb, typ);

                    listeKinder.add(davTyp);
                }
            }

            Collections.sort(listeKinder, COMPARATOR_SO_PID);
            _mapKBKinder.put(kb.getSystemObjekt().getPidOrId(), listeKinder);
        }

        // --------------------------------------------------------
        _listeTyp = new ArrayList<DaVObjektTyp>();

        List<SystemObject> listeSo;

        try {
            listeSo = _davKonf.getType("typ.typ").getElements();
        }
        catch (Exception e) {
            listeSo = new ArrayList<SystemObject>();
        }

        for (SystemObject so : listeSo) {
            DaVObjektTyp davTyp = new DaVObjektTyp(null, (SystemObjectType) so);

            _listeTyp.add(davTyp);
        }

        Collections.sort(_listeTyp, COMPARATOR_SO_PID);
    }

    /**
     * Initialisiere alle ZaehlStellen und map Strae->ZaehlStellen.
     */
    private void initListeZaehlStellen() {
        _listeZaehlStellen      = new ArrayList<ZaehlStelle>();
        _mapStrasseZaehlStellen = new HashMap<String, List<ZaehlStelle>>();

        // Alle Systemobjekte vom Typ LangZeitZhlstelle:
        List<SystemObject> lisateZsSO;

        try {
            lisateZsSO = _davKonf.getType("typ.langZeitZhlstelle").getElements();
        }
        catch (Exception e) {
            lisateZsSO = new ArrayList<SystemObject>();
        }

        //
        AttributeGroup          atgZs     = _davKonf.getAttributeGroup("atg.langZeitZhlstelle");
        Aspect                  aspZs     = _davKonf.getAspect("asp.eigenschaften");
        Map<Data, SystemObject> mapKonfZs = new HashMap<Data, SystemObject>();

        for (SystemObject zsSo : lisateZsSO) {
            List<SystemObject> listeSo = new ArrayList<SystemObject>();

            listeSo.add(zsSo);

            Data[] konfD = _davKonf.getConfigurationData(listeSo, atgZs, aspZs);

            for (Data konfData : konfD) {
                mapKonfZs.put(konfData, zsSo);
            }
        }

        //
        List<SystemObject> listeStrassenSo;

        try {

            // Alle Sytemojbkete vom Typ Strasse:
            listeStrassenSo = _davKonf.getType("typ.strae").getElements();
        }
        catch (Exception e) {
            listeStrassenSo = new ArrayList<SystemObject>();
        }

        for (SystemObject strasse : listeStrassenSo) {
            List<ZaehlStelle> listeZs = new ArrayList<ZaehlStelle>();

            for (Data konfData : mapKonfZs.keySet()) {
                if (konfData.getReferenceValue("Strae").getSystemObject().equals(strasse)) {
                    ZaehlStelle zs = new ZaehlStelle(strasse, mapKonfZs.get(konfData));

                    listeZs.add(zs);
                    _listeZaehlStellen.add(zs);
                }
            }

            Collections.sort(listeZs, COMPARATOR_SO_PID);
            _mapStrasseZaehlStellen.put(strasse.getPidOrId(), listeZs);
        }

        //
        Collections.sort(_listeZaehlStellen, COMPARATOR_SO_PID);
    }

    /**
     * Initialisiere das Map aller Fahrbahnen.
     */
    private void initMapFahrbahnen() {
        Fahrbahnen fs2;
        String     quelle;

        _mapFahrbahnen = new HashMap<String, Fahrbahnen>();

        for (ZaehlStelle zs : _listeZaehlStellen) {
            for (AuswerteQuerschnitt awq : zs.getArAWQ()) {
                Messquerschnitt mq = awq.getMessquerschnitt();

                for (Fahrbahnen fs : mq.getArFahrbahnen()) {
                    quelle = fs.getFsQuellePid();

                    if (quelle != null) {
                        fs2 = _mapFahrbahnen.put(quelle, fs);

                        if (fs2 != null) {
                            Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, String.format(  //
                                "Fahrbahnen mit Fahrstreifenquelle <%s> exestiert mehrmals", quelle));
                        }
                    }
                }
            }
        }
    }

    /**
     * Initialisiere rekursiv das Map aller Gerte.
     *
     * @param listeGeraete Liste der Gerte.
     */
    private void initMapTlsGeraete(Object[] listeGeraete) {
        Geraet g2;

        for (Object ob : listeGeraete) {

            // Geraet:
            if (ob instanceof Geraet) {
                Geraet geraet = (Geraet) ob;

                g2 = _mapTlsGeraete.put(geraet.getSystemObjekt().getPidOrId(), geraet);

                if (g2 != null) {
                    Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, String.format(  //
                        "Geraet mit Pid <%s> wurde mehrmals erstellt", geraet.getSystemObjekt().getPidOrId()));
                }

                // Rekursion:
                initMapTlsGeraete(geraet.ermittleKinder());
            }

            // EAK:
            else if (ob instanceof Eak) {
                Eak eak = (Eak) ob;

                for (De de : eak.getListeDe()) {
                    _mapDe.put(de.getSystemObjekt().getPidOrId(), de);
                }
            }

            // Unbekannt:
            else {
                Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, String.format(  //
                    "Unbekanntes Objekt <%s> wurde beim Initialisieren des Maps aller Gerte entdeckt", ob));
            }
        }
    }

    /**
     * Initialisiere alle Straen.
     */
    private void initStrassen() {
        _listeStrassen = new ArrayList<Strasse>();
        _mapStrassen   = new HashMap<String, Strasse>();

        List<SystemObject> listeStrassenSo;

        try {

            // Alle Sytemojbkete vom Typ Strasse:
            listeStrassenSo = _davKonf.getType("typ.strae").getElements();
        }
        catch (Exception e) {
            listeStrassenSo = new ArrayList<SystemObject>();
        }

        for (SystemObject soStrasse : listeStrassenSo) {
            Strasse strasse = new Strasse(soStrasse);

            _listeStrassen.add(strasse);
            _mapStrassen.put(soStrasse.getPidOrId(), strasse);
        }

        Collections.sort(_listeStrassen, COMPARATOR_SO_PID);
    }

    /**
     * Pause bis dem Ende der Initialisierung
     * s.{@link #_istInitialisierungAbgeschlossen}.
     */
    private void pauseBisEndeDerInitialisierung() {
        while (!_istInitialisierungAbgeschlossen) {
            try {
                Thread.sleep(100);
            }
            catch (Exception e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 25.02.13
