/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.buv.inovat.lib.io;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

//~ JDK IMPORTE ===============================================================

import java.io.*;

import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Ermglicht den schreibenden Zugriff auf Excel-Tabellen im *.csv-Format.
 *
 * @author inovat
 * @author Dipl.-Inform. Joachim Collasius (JC)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class CsvSchreiber {

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Private ----------------------------------------------------------------

    /** Trennzeichen (i.d.R. ';'), durch welches einzelne Zellen innerhalb einer Zeile getrennt sind. */
    private char _begrenzungsZeichen;

    /** Instanz des Writer Objekts, mit dem in die CSV Datei geschrieben wird. */
    private BufferedWriter _csvWriter;

    /** Kopfzeile der CSV Datei, die den Inhalt der Spalten definiert. */
    private List<String> _kopfZeile;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /**
     * Konstruktor
     *
     * @param schreiber die Reprsentation des Schreibers, der die Daten gepuffert in die CSV Datei schreibt.
     * @param begrenzungsZeichen das Zeichen, das die einzelnen Werte einer Zeile in der CSV Datei begrenzt.
     */
    public CsvSchreiber(Writer schreiber, char begrenzungsZeichen) {
        _begrenzungsZeichen = begrenzungsZeichen;
        _csvWriter          = new BufferedWriter(schreiber);
    }

    //~ METHODEN ==============================================================

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /** Schliet die *.csv-Datei. */
    public void close() {
        try {
            _csvWriter.close();
        }
        catch (IOException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Hngt eine Zeile, bestehend aus den einzelnen Feldern des CSV Formats,
     * jeweils gepuffert am Ende einer bereits existierenden CSV Datei an.
     * Falls fr ein Feld kein Wert gesetzt ist, wird ein leerer String ausgegeben.
     * Die Felder jeder Zeile werden jeweils durch das Begrenzungszeichen voneinander abgetrennt.
     * Am Ende jeder Zeile folgt ein Zeilenumbruch und anschlieend wird der Puffer in die CSV Datei geschrieben.
     *
     * @param zeile ein Map Objekt als Reprsentation einer Zeile des CSV Formats,
     * bestehend jeweils aus dem Feldnamen einer Spalte des CSV Formats als Schlssel und dem zugehrigen Wert des Feldes.
     */
    public void haengeZeileAn(Map<String, String> zeile) {
        try {
            for (String spalte : _kopfZeile) {
                String zelle = zeile.get(spalte);

                if (zelle == null) {
                    _csvWriter.write("");
                } else {
                    _csvWriter.write(zelle);
                }

                _csvWriter.append(_begrenzungsZeichen);
            }

            _csvWriter.newLine();
            _csvWriter.flush();
        }
        catch (IOException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Schreibt die einzelnen Felder, die die Spalten des CSV Formats definieren, gepuffert als Kopfzeile in die CSV Datei.
     * Die Felder der Kopfzeile werden jeweils durch das Begrenzungszeichen voneinander abgetrennt.
     * Am Ende der Kopfzeile folgt ein Zeilenumbruch und anschlieend wird der Puffer in die CSV Datei geschrieben.
     */
    public void schreibeKopfZeile() {
        try {
            for (String spalte : _kopfZeile) {
                _csvWriter.write(spalte);
                _csvWriter.write(_begrenzungsZeichen);
            }

            _csvWriter.newLine();
            _csvWriter.flush();
        }
        catch (IOException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Schreibt die Werte der einzelnen Felder, die die Spalten des CSV Formats definieren, gepuffert als Zeile in die CSV Datei.
     * Falls fr ein Feld kein Wert gesetzt ist, wird ein leerer String ausgegeben.
     * Die Felder jeder Zeile werden jeweils durch das Begrenzungszeichen voneinander abgetrennt.
     * Am Ende jeder Zeile folgt ein Zeilenumbruch und anschlieend wird der Puffer in die CSV Datei geschrieben.
     *
     * @param zeile ein Map Objekt als Reprsentation einer Zeile des CSV Formats,
     * bestehend jeweils aus dem Feldnamen einer Spalte des CSV Formats als Schlssel und dem zugehrigen Wert des Feldes.
     */
    public void schreibeZeile(Map<String, String> zeile) {
        try {
            for (String spalte : _kopfZeile) {
                String zelle = zeile.get(spalte);

                if (zelle == null) {
                    _csvWriter.write("");
                } else {
                    _csvWriter.write(zelle);
                }

                _csvWriter.write(_begrenzungsZeichen);
            }

            _csvWriter.newLine();
            _csvWriter.flush();
        }
        catch (IOException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    // ------------------------------------------------------------------------
    // GETTER / SETTER METHODEN
    // ------------------------------------------------------------------------

    /**
     * Setzt die Kopfzeile, die die einzelnen Felder der CSV Datei definiert.
     *
     * @param kopfZeile die Liste der einzelnen Felder des CSV Formats.
     */
    public void setzeKopfZeile(List<String> kopfZeile) {
        _kopfZeile = kopfZeile;

        // TODO evtl. separat definierte Kopfzeile in die CSV Datei ausgeben
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.05.09
