/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTM;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.navigation.Activator;
import de.inovat.buv.projektlib.speicher.GrundEinstellungen;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Einstellungen fr die Tabellenverwaltung, die im XML Format gespeichert werden.
 * Die Klasse ist als Singelton realisiert.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public final class AlleTabellenVerwaltungen {

    /**
     * Stellt fest, ob die Initialisierung der Instanz abgeschlossen ist (s.{@link #AlleTabellenVerwaltungen()}).
     *
     * WICHTIG:
     *
     * Die Methode {@link #getTabellenVerwaltungInstanz()} MUSS auf das Ende der Initialisierung warten.
     */
    private static boolean _istInitialisierungAbgeschlossen = false;

    /**
     * Die Menge die als Listener auf Sortierungnderungen registrierten Klassen.
     * Diese Klassen implementieren ISortierungAenderung.
     */
    private static final Set<ITabellenVerwaltungAenderung> mengeListener = new HashSet<ITabellenVerwaltungAenderung>();

    /**
     * Einzige Instanz der Klasse fr die TabellenVerwaltung.
     */
    private static AlleTabellenVerwaltungen _instanz = new AlleTabellenVerwaltungen();
    public static boolean                   _klasseLaden;

    //~ FELDER ================================================================

    /**
     * Map der definierten TabellenVerwaltung.
     * Key: Name der TabellenVerwaltung.
     */
    private final Map<String, TabellenVerwaltung> _tabellenVerwaltungMap = new Hashtable<String, TabellenVerwaltung>();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     *
     * Erzeugt eine Instanz der Klasse AlleTabellenVerwaltungen.
     *
     */
    private AlleTabellenVerwaltungen() {

        // -------------------------------------------------------------------
        // WICHTIG
        // Die Initialisierung der Tabellendarstellungen luft langsam ===>
        // wird in JOB ausgelagert.
        // ===>
        // Die Methode {@link #getTabellenVerwaltungInstanz()} MUSS auf
        // das Ende der Initialisierung warten.
        Job job = new Job("Initialisiere alle Tabellendarstellungen...") {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                ladeAlleTabellenVerwaltungen();

                //
                _istInitialisierungAbgeschlossen = true;

                return Status.OK_STATUS;
            }
        };

        job.schedule();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode fgt eine neue Tabellenverwaltung in tabellenVerwaltung ein.
     * @param tabellenVerwaltung - Tabellenverwaltung.
     */
    public void addTabellenVerwaltung(TabellenVerwaltung tabellenVerwaltung) {
        synchronized (_tabellenVerwaltungMap) {
            _tabellenVerwaltungMap.put(tabellenVerwaltung.getTabellenVerwaltungName(), tabellenVerwaltung);
        }
    }

    /**
     * Registriert die Klasse als Listener auf TabellenVeraltungnderungen.
     * @param klasse - die ITabelleVerwaltungAenderung implementierte Klasse.
     */
    public static void addTabellenVerwaltungListener(ITabellenVerwaltungAenderung klasse) {
        synchronized (mengeListener) {
            mengeListener.add(klasse);
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt das sortierte Array der Namen von definierten TabellenVerwaltungen zurck,
     * oder leeres StringArray, falls keine TabellenVerwaltungen definiert sind.
     * @return Sortiertes Array der Namen der definierten TabellenVerwaltungen, oder leeres String Array, falls keine TabellenVerwaltungen
     * definiert sind.
     */
    public String[] getTabellenVerwaltungArray() {
        String[] ar;

        synchronized (_tabellenVerwaltungMap) {
            ar = _tabellenVerwaltungMap.keySet().toArray(new String[_tabellenVerwaltungMap.keySet().size()]);
        }

        Arrays.sort(ar);

        return ar;
    }

    /**
     * Gibt einzige Instanz der Klasse zurck.
     *
     * @return Instanz der Klasse.
     */
    public static AlleTabellenVerwaltungen getTabellenVerwaltungInstanz() {
        pauseBisEndeDerInitialisierung();

        return _instanz;
    }

    /**
     * Gibt das TabellenVerwaltungen Objekt mit angegebenem Namen zurck
     * oder null, falls dieses nicht exisitert.
     * @param tabellenVerwaltungsName Tabellenverwaltungsname.
     * @return Tabellenverwaltung oder null.
     */
    public TabellenVerwaltung getTabellenVerwaltungsObjekt(String tabellenVerwaltungsName) {
        return _tabellenVerwaltungMap.get(tabellenVerwaltungsName);
    }

    //~ METHODEN ==============================================================

    /**
     * Lade alle TabellenVerwaltungen.
     */
    private void ladeAlleTabellenVerwaltungen() {

        // Ermittle die String-Representation des Maps aus dem Speicher:
        String mapString = GrundEinstellungen.ermittleParameterWertFuerAlleBenutzer(KonstantenGTM.ALLE_TABELLEN_VERWALTUNGEN);

        if (mapString != null) {

            // Dekodiere den Parameter:
            try {
                ByteArrayInputStream input     = new ByteArrayInputStream(mapString.getBytes());
                XMLDecoder           xml       = new XMLDecoder(input);
                Object               mapObjekt = xml.readObject();

                xml.close();

                if (mapObjekt instanceof Map) {
                    try {
                        synchronized (_tabellenVerwaltungMap) {
                            _tabellenVerwaltungMap.putAll((Hashtable<String, TabellenVerwaltung>) mapObjekt);
                        }
                    }
                    catch (Exception e) {
                        Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Sortierungen knnen nicht geladen werden!", e));
                    }
                }
            }
            catch (Exception e) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Sortierungen knnen nicht geladen werden!", e));
            }
        }
    }

    /**
     * Bei nderung in der Tabellenverwaltung Liste (beim Aufruf der Methode speicherXML())
     * wird die Methode sortierungGeaendert() fr jede als Listener registrierte Klasse aufgerufen.
     */
    private static void listenerBenachrichtigen() {
        synchronized (mengeListener) {
            for (ITabellenVerwaltungAenderung klasse : mengeListener) {
                klasse.tabellenVerwaltungGeandert();
            }
        }
    }

    /**
     * Lscht das Tabellenverwaltungs Objekt mit dem Namen tabellenVerwaltungName aus der
     * Liste der Tabellenverwaltungen.
     * @param tabellenVerwaltungName Name der zu lschenden Tabellenverwaltung.
     */
    public void loescheTabellenVerwaltung(String tabellenVerwaltungName) {
        synchronized (_tabellenVerwaltungMap) {
            _tabellenVerwaltungMap.remove(tabellenVerwaltungName);
        }
    }

    /**
     * Pause bis dem Ende der Initialisierung
     * s.{@link #_istInitialisierungAbgeschlossen}.
     */
    private static void pauseBisEndeDerInitialisierung() {
        while (!_istInitialisierungAbgeschlossen) {
            try {
                Thread.sleep(100);
            }
            catch (Exception e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }
    }

    /**
     * Die Klasse wird vom Listener auf TabellenVerwaltungsnderungen abgemeldet.
     * @param klasse - die abzumeldende Klasse.
     */
    public static void removeTabellenVerwaltungListener(ITabellenVerwaltungAenderung klasse) {
        synchronized (mengeListener) {
            mengeListener.remove(klasse);
        }
    }

    /**
     * Speichert den Inhalt von _tabellenVerwaltungsMap in XML format.
     */
    public void speichereAlleTabellenVerwaltungen() {

        // Kodiere _tabellenVerwaltungsMap ber XML Encoder:
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        XMLEncoder            xml = new XMLEncoder(out);

        synchronized (_tabellenVerwaltungMap) {
            xml.writeObject(_tabellenVerwaltungMap);
        }

        xml.close();

        //
        boolean gespeichert = GrundEinstellungen.speichereParameterFuerAlleBenutzer(KonstantenGTM.ALLE_TABELLEN_VERWALTUNGEN, out.toString(), "Speichern von Tabellenverwaltungen");

        if (!gespeichert) {
            MessageDialog.openError(PlatformUI.getWorkbench().getDisplay().getActiveShell(), "Fehler beim Speichern", "Es ist ein Fehler beim Speichern der Sortierungen aufgetretten. Details siehe unten im Meldungsfenster.");
        }

        // Benachrichtige Listener ber die nderunngen in _sortierungMap:
        listenerBenachrichtigen();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 05.11.13
