/*
 * @(#)KonvertorUTMtoWGS84.java   15.09.11
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.visualisierung.karte;

//~ KLASSEN ===================================================================

/**
 * Klasse liefert die statischen Methode fr die Konvertierung der Koordinaten
 * zwischen UTM und WGS84Dec.
 *
 * <p>
 * Siehe Seitenquelltext: http://home.hiwaay.net/~taylorc/toolbox/geography/geoutm.html
 * (Rechtlicher Hinweis: The JavaScript source code in this document may be copied and reused without restriction.)
 * <p>
 * Eine Seite zum Testen der Projektion mit Verlinkung zu Google Maps:
 * http://pages.globetrotter.net/roule/utmgoogle.htm
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 296 $ / $Date: 2012-11-08 11:06:28 +0100 (Do, 08 Nov 2012) $ / ($Author: LG $)
 */
public class KoordinatenKonvertor {
    private static final double pi             = 3.14159265358979;
    private static final double sm_a           = 6378137.0;
    private static final double sm_b           = 6356752.314;
    private static final double UTMScaleFactor = 0.9996;

    /** Zone fr die Niedersachsen. */
    private static final int zone = 32;

    /** Niedersachsen liegt in der nrdlichen Hemisphre. */
    private static final boolean southhemi = false;

    //~ METHODEN ==============================================================

    /**
     * Computes the ellipsoidal distance from the equator to a point at a given
     * latitude.
     *
     * Reference: Hoffmann-Wellenhof, B., Lichtenegger, H., and Collins, J., GPS:
     * Theory and Practice, 3rd ed. New York: Springer-Verlag Wien, 1994.
     *
     * @param phi Latitude of the point, in radians.
     * Globals: sm_a - Ellipsoid model major axis. sm_b - Ellipsoid model minor axis.
     *
     * @return The ellipsoidal distance of the point from the equator, in meters.
     */
    private static double ArcLengthOfMeridian(double phi) {
        double alpha, beta, gamma, delta, epsilon, n;
        double result;

        /* Precalculate n */
        n = (sm_a - sm_b) / (sm_a + sm_b);

        /* Precalculate alpha */
        alpha = ((sm_a + sm_b) / 2.0)  //
            * (1.0 + (Math.pow(n, 2.0) / 4.0) + (Math.pow(n, 4.0) / 64.0));

        /* Precalculate beta */
        beta = (-3.0 * n / 2.0) + (9.0 * Math.pow(n, 3.0) / 16.0)  //
            + (-3.0 * Math.pow(n, 5.0) / 32.0);

        /* Precalculate gamma */
        gamma = (15.0 * Math.pow(n, 2.0) / 16.0)  //
            + (-15.0 * Math.pow(n, 4.0) / 32.0);

        /* Precalculate delta */
        delta = (-35.0 * Math.pow(n, 3.0) / 48.0)  //
            + (105.0 * Math.pow(n, 5.0) / 256.0);

        /* Precalculate epsilon */
        epsilon = (315.0 * Math.pow(n, 4.0) / 512.0);

        /* Now calculate the sum of the series and return */
        result = alpha                                                             //
            * (phi + (beta * Math.sin(2.0 * phi)) + (gamma * Math.sin(4.0 * phi))  //
                + (delta * Math.sin(6.0 * phi)) + (epsilon * Math.sin(8.0 * phi)));

        return result;
    }

    /**
     * Converts degrees to radians.
     *
     * @param deg degrees.
     *
     * @return radians.
     */
    private static double DegToRad(double deg) {
        return (deg / 180.0 * pi);
    }

    /**
     * Ermittle die UTM-Zone aus der angegebene WGS84Dec-X-Koordinate.
     *
     * @param x WGS-X-Koordinate (longitude).
     *
     * @return UTM-Zone oder null, falls _Fehler aufgetretten sind.
     */
    public static int ermittleUTMZone(double x) {
        try {
            return (int) Math.floor((x + 180.0) / 6) + 1;
        }
        catch (Exception e) {
            return -1;
        }
    }

    /**
     * Computes the footpoint latitude for use in converting transverse
     * Mercator coordinates to ellipsoidal coordinates.
     *
     * Reference: Hoffmann-Wellenhof, B., Lichtenegger, H., and Collins, J.,
     *   GPS: Theory and Practice, 3rd ed.  New York: Springer-Verlag Wien, 1994.
     *
     * @param y The UTM northing coordinate, in meters.
     *
     * @return The footpoint latitude, in radians.
     */
    private static double FootpointLatitude(double y) {
        double y_, alpha_, beta_, gamma_, delta_, epsilon_, n;
        double result;

        /* Precalculate n (Eq. 10.18) */
        n = (sm_a - sm_b) / (sm_a + sm_b);

        /* Precalculate alpha_ (Eq. 10.22) */
        /* (Same as alpha in Eq. 10.17) */
        alpha_ = ((sm_a + sm_b) / 2.0)  //
            * (1 + (Math.pow(n, 2.0) / 4) + (Math.pow(n, 4.0) / 64));

        /* Precalculate y_ (Eq. 10.23) */
        y_ = y / alpha_;

        /* Precalculate beta_ (Eq. 10.22) */
        beta_ = (3.0 * n / 2.0) + (-27.0 * Math.pow(n, 3.0) / 32.0)  //
            + (269.0 * Math.pow(n, 5.0) / 512.0);

        /* Precalculate gamma_ (Eq. 10.22) */
        gamma_ = (21.0 * Math.pow(n, 2.0) / 16.0)  //
            + (-55.0 * Math.pow(n, 4.0) / 32.0);

        /* Precalculate delta_ (Eq. 10.22) */
        delta_ = (151.0 * Math.pow(n, 3.0) / 96.0)  //
            + (-417.0 * Math.pow(n, 5.0) / 128.0);

        /* Precalculate epsilon_ (Eq. 10.22) */
        epsilon_ = (1097.0 * Math.pow(n, 4.0) / 512.0);

        /* Now calculate the sum of the series (Eq. 10.21) */
        result = y_ + (beta_ * Math.sin(2.0 * y_))  //
            + (gamma_ * Math.sin(4.0 * y_))         //
                + (delta_ * Math.sin(6.0 * y_))     //
                    + (epsilon_ * Math.sin(8.0 * y_));

        return result;
    }

    /**
     * Ermittle, ob die Hemisphre fr die angegebene WGS84Dec-Y-Koordinate sdlich ist.
     *
     * @param y WGS-Y-Koordinate (latitude).
     *
     * @return true, falls Hemisphre fr die angegebene WGS84Dec-Y-Koordinate
     * sdlich ist, false - sonst.
     */
    public static boolean istHemisphaereSued(double y) {
        return y < 0;
    }

    /**
     * Converts a latitude/longitude pair to x and y coordinates in the Universal
     * Transverse Mercator projection.
     *
     * @param lat Latitude of the point, in radians.
     * @param lon Longitude of the point, in radians.
     * @param zone UTM zone to be used for calculating values for x
     * and y. If zone is less than 1 or greater than 60, the routine will
     * determine the appropriate zone from the value of lon.
     *
     * @return Array 2-element array where the UTM x and y values will be stored.
     */
    private static double[] LatLonToUTMXY(double lat, double lon, int zone) {
        double[] xy = MapLatLonToXY(lat, lon, UTMCentralMeridian(zone));

        /* Adjust easting and northing for UTM system. */
        xy[0] = xy[0] * UTMScaleFactor + 500000.0;
        xy[1] = xy[1] * UTMScaleFactor;

        if (xy[1] < 0.0) {
            xy[1] = xy[1] + 10000000.0;
        }

        return xy;
    }

    /**
     * TODO Test schreiben...
     * Main fr Testzwecke.
     *
     * @param args Aufrufparameter.
     */
    public static void main(String[] args) {
        System.out.println("---------------------------------");
        System.out.println("utm --> wgs");

        // double   x   = 445952.38273735234;
        // double   y   = 5888726.779505275;
        //
        // double   x   = 338534.3508; // links
        double x = 338534.3508 + 226139.375;  // rechts

        //
        // double   y   = 5527430.047-272494.375; // unten
        double   y   = 5527430.047;  // oben
        double[] wgs = utm2wgs(x, y);

        //
        System.out.println("utm x = " + x);
        System.out.println("utm y = " + y);
        System.out.println("wgs x = " + wgs[0]);
        System.out.println("wgs y = " + wgs[1]);

        // ----------------------------------------------
        System.out.println("---------------------------------");
        System.out.println("wgs --> utm");

        double lon = wgs[0];
        double lat = wgs[1];

        //
        double[] xy = wgs2utm(lon, lat);

        System.out.println(xy[0]);
        System.out.println(xy[1]);
        System.out.println("zone = " + ermittleUTMZone(lon));
        System.out.println("Sd = " + istHemisphaereSued(lat));
    }

    /**
     * Converts a latitude/longitude pair to x and y coordinates in the Transverse
     * Mercator projection. Note that Transverse Mercator is not the same as UTM;
     * a scale factor is required to convert between them.
     *
     * Reference: Hoffmann-Wellenhof, B., Lichtenegger, H., and Collins, J., GPS:
     * Theory and Practice, 3rd ed. New York: Springer-Verlag Wien, 1994.
     *
     * @param phi Latitude of the point, in radians.
     * @param lambda Longitude of the point, in radians.
     * @param lambda0 Longitude of the central meridian to be used, in radians.
     *
     * @return Array 2-element array containing the x and y coordinates of the computed point.
     */
    private static double[] MapLatLonToXY(double phi, double lambda, double lambda0) {
        double N, nu2, ep2, t, t2, l;
        double l3coef, l4coef, l5coef, l6coef, l7coef, l8coef;

        // double tmp; wird nicht bentigt

        /* Precalculate ep2 */
        ep2 = (Math.pow(sm_a, 2.0) - Math.pow(sm_b, 2.0)) / Math.pow(sm_b, 2.0);

        /* Precalculate nu2 */
        nu2 = ep2 * Math.pow(Math.cos(phi), 2.0);

        /* Precalculate N */
        N = Math.pow(sm_a, 2.0) / (sm_b * Math.sqrt(1 + nu2));

        /* Precalculate t */
        t  = Math.tan(phi);
        t2 = t * t;

        // tmp = (t2 * t2 * t2) - Math.pow(t, 6.0); wird nicht bentigt

        /* Precalculate l */
        l = lambda - lambda0;

        /*
         * Precalculate coefficients for l**n in the equations below so a normal
         * human being can read the expressions for easting and northing -- l**1 and
         * l**2 have coefficients of 1.0
         */
        l3coef = 1.0 - t2 + nu2;
        l4coef = 5.0 - t2 + 9 * nu2 + 4.0 * (nu2 * nu2);
        l5coef = 5.0 - 18.0 * t2 + (t2 * t2) + 14.0 * nu2 - 58.0 * t2 * nu2;
        l6coef = 61.0 - 58.0 * t2 + (t2 * t2) + 270.0 * nu2 - 330.0 * t2 * nu2;
        l7coef = 61.0 - 479.0 * t2 + 179.0 * (t2 * t2) - (t2 * t2 * t2);
        l8coef = 1385.0 - 3111.0 * t2 + 543.0 * (t2 * t2) - (t2 * t2 * t2);

        //
        double[] xy = new double[2];

        /* Calculate easting (x) */
        xy[0] = N                                                                                 //
            * Math.cos(phi)                                                                       //
                * l                                                                               //
                    + (N / 6.0 * Math.pow(Math.cos(phi), 3.0) * l3coef * Math.pow(l, 3.0))        //
                        + (N / 120.0 * Math.pow(Math.cos(phi), 5.0) * l5coef * Math.pow(l, 5.0))  //
                            + (N / 5040.0 * Math.pow(Math.cos(phi), 7.0) * l7coef * Math          //
                                .pow(l, 7.0));

        /* Calculate northing (y) */
        xy[1] = ArcLengthOfMeridian(phi)                                                          //
            + (t / 2.0 * N * Math.pow(Math.cos(phi), 2.0) * Math.pow(l, 2.0))                     //
                + (t / 24.0 * N * Math.pow(Math.cos(phi), 4.0) * l4coef * Math.pow(l, 4.0))       //
                    + (t / 720.0 * N * Math.pow(Math.cos(phi), 6.0) * l6coef * Math.pow(l, 6.0))  //
                        + (t / 40320.0 * N * Math.pow(Math.cos(phi), 8.0) * l8coef * Math.pow(l, 8.0));

        return xy;
    }

    /**
     * Converts x and y coordinates in the Transverse Mercator projection to
     * a latitude/longitude pair.  Note that Transverse Mercator is not
     * the same as UTM; a scale factor is required to convert between them.
     *
     * Reference: Hoffmann-Wellenhof, B., Lichtenegger, H., and Collins, J.,
     *   GPS: Theory and Practice, 3rd ed.  New York: Springer-Verlag Wien, 1994.
     *
     * Remarks:
     *   The local variables Nf, nuf2, tf, and tf2 serve the same purpose as
     *   N, nu2, t, and t2 in MapLatLonToXY, but they are computed with respect
     *   to the footpoint latitude phif.
     *
     *   x1frac, x2frac, x2poly, x3poly, etc. are to enhance readability and
     *   to optimize computations.
     *
     * @param x The easting of the point, in meters.
     * @param y The northing of the point, in meters.
     * @param lambda0 Longitude of the central meridian to be used, in radians.
     *
     * @return Array 2-element containing the latitude and longitude in radians.
     */
    private static double[] MapXYToLatLon(double x, double y, double lambda0) {
        double phif, Nf, Nfpow, nuf2, ep2, tf, tf2, tf4, cf;
        double x1frac, x2frac, x3frac, x4frac, x5frac, x6frac, x7frac, x8frac;
        double x2poly, x3poly, x4poly, x5poly, x6poly, x7poly, x8poly;

        /* Get the value of phif, the footpoint latitude. */
        phif = FootpointLatitude(y);

        /* Precalculate ep2 */
        ep2 = (Math.pow(sm_a, 2.0) - Math.pow(sm_b, 2.0)) / Math.pow(sm_b, 2.0);

        /* Precalculate cos (phif) */
        cf = Math.cos(phif);

        /* Precalculate nuf2 */
        nuf2 = ep2 * Math.pow(cf, 2.0);

        /* Precalculate Nf and initialize Nfpow */
        Nf    = Math.pow(sm_a, 2.0) / (sm_b * Math.sqrt(1 + nuf2));
        Nfpow = Nf;

        /* Precalculate tf */
        tf  = Math.tan(phif);
        tf2 = tf * tf;
        tf4 = tf2 * tf2;

        /* Precalculate fractional coefficients for x**n in the equations
           below to simplify the expressions for latitude and longitude. */
        x1frac = 1.0 / (Nfpow * cf);
        Nfpow  *= Nf;  /* now equals Nf**2) */
        x2frac = tf / (2.0 * Nfpow);
        Nfpow  *= Nf;  /* now equals Nf**3) */
        x3frac = 1.0 / (6.0 * Nfpow * cf);
        Nfpow  *= Nf;  /* now equals Nf**4) */
        x4frac = tf / (24.0 * Nfpow);
        Nfpow  *= Nf;  /* now equals Nf**5) */
        x5frac = 1.0 / (120.0 * Nfpow * cf);
        Nfpow  *= Nf;  /* now equals Nf**6) */
        x6frac = tf / (720.0 * Nfpow);
        Nfpow  *= Nf;  /* now equals Nf**7) */
        x7frac = 1.0 / (5040.0 * Nfpow * cf);
        Nfpow  *= Nf;  /* now equals Nf**8) */
        x8frac = tf / (40320.0 * Nfpow);

        /* Precalculate polynomial coefficients for x**n.
           -- x**1 does not have a polynomial coefficient. */
        x2poly = -1.0 - nuf2;
        x3poly = -1.0 - 2 * tf2 - nuf2;
        x4poly = 5.0 + 3.0 * tf2 + 6.0 * nuf2 - 6.0 * tf2 * nuf2  //
            - 3.0 * (nuf2 * nuf2) - 9.0 * tf2 * (nuf2 * nuf2);
        x5poly = 5.0 + 28.0 * tf2 + 24.0 * tf4 + 6.0 * nuf2 + 8.0 * tf2 * nuf2;
        x6poly = -61.0 - 90.0 * tf2 - 45.0 * tf4 - 107.0 * nuf2   //
            + 162.0 * tf2 * nuf2;
        x7poly = -61.0 - 662.0 * tf2 - 1320.0 * tf4 - 720.0 * (tf4 * tf2);
        x8poly = 1385.0 + 3633.0 * tf2 + 4095.0 * tf4 + 1575 * (tf4 * tf2);

        //
        double[] philambda = new double[2];

        /* Calculate latitude */
        philambda[0] = phif + x2frac * x2poly * (x * x)  //
            + x4frac * x4poly * Math.pow(x, 4.0)         //
                + x6frac * x6poly * Math.pow(x, 6.0)     //
                    + x8frac * x8poly * Math.pow(x, 8.0);

        /* Calculate longitude */
        philambda[1] = lambda0 + x1frac * x           //
            + x3frac * x3poly * Math.pow(x, 3.0)      //
                + x5frac * x5poly * Math.pow(x, 5.0)  //
                    + x7frac * x7poly * Math.pow(x, 7.0);

        return philambda;
    }

    /**
     * Converts radians to degrees.
     *
     * @param rad radians.
     *
     * @return degrees.
     */
    private static double RadToDeg(double rad) {
        return (rad / pi * 180.0);
    }

    /**
     * Konvertiere die angegebenen UTM-Koordinaten fr Niedersachsen in WGS84Dec.
     *
     * @param x UTM-X-Koordinate (The easting of the point, in meters).
     * @param y UTM-Y-Koordinate (The northing of the point, in meters).
     *
     * @return Koordinaten in WGS84Dec {x, y}, wobei x = longitude y = latitude.
     * Falls _Fehler aufgetretten sind, wird null zurckgegeben.
     */
    public static double[] utm2wgs(double x, double y) {
        try {
            double[] xy = UTMXYToLatLon(x, y, zone, southhemi);

            //
            double[] wgs = new double[2];

            wgs[0] = RadToDeg(xy[1]);
            wgs[1] = RadToDeg(xy[0]);

            return wgs;
        }
        catch (Exception e) {
            return null;
        }
    }

    /**
     * Determines the central meridian for the given UTM zone.
     *
     * @param zone An integer value designating the UTM zone, range [1,60].
     *
     * @return
     *   The central meridian for the given UTM zone, in radians, or zero
     *   if the UTM zone parameter is outside the range [1,60].
     *   Range of the central meridian is the radian equivalent of [-177,+177].
     */
    private static double UTMCentralMeridian(int zone) {
        return DegToRad(-183.0 + (zone * 6.0));
    }

    /**
     * Converts x and y coordinates in the Universal Transverse Mercator
     * projection to a latitude/longitude pair.
     *
     * @param x The easting of the point, in meters.
     * @param y The northing of the point, in meters.
     * @param zone The UTM zone in which the point lies.
     * @param southhemi True if the point is in the southern hemisphere, false otherwise.
     *
     * @return Array 2-element array containing the latitude and longitude of the point, in radians.
     */
    private static double[] UTMXYToLatLon(double x, double y, int zone, boolean southhemi) {
        double cmeridian;

        x -= 500000.0;
        x /= UTMScaleFactor;

        /* If in southern hemisphere, adjust y accordingly. */
        if (southhemi) {
            y -= 10000000.0;
        }

        y         /= UTMScaleFactor;
        cmeridian = UTMCentralMeridian(zone);

        return MapXYToLatLon(x, y, cmeridian);
    }

    /**
     * Konvertiere die angegebenen WGS84Dec-Koordinaten in UTM-Koordinaten.
     *
     * @param x WGS-X-Koordinate (longitude).
     * @param y WGS-Y-Koordinate (latitude).
     *
     * @return UTM-Koordinaten{x, y}.
     * x = The easting of the point, in meters,
     * y = The northing of the point, in meters.
     *
     * Falls _Fehler aufgetretten sind, wird null zurckgegeben.
     */
    public static double[] wgs2utm(double x, double y) {
        try {
            int zone = (int) Math.floor((x + 180.0) / 6) + 1;

            return LatLonToUTMXY(DegToRad(y), DegToRad(x), zone);
        }
        catch (Exception e) {
            return null;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.11.12
