/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.speicher;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;

import de.inovat.buv.projektlib.speicher.einstellungen.SpeicherArtUndOrt;
import de.inovat.buv.projektlib.speicher.einstellungen.SpeicherFunktionen;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die gespeicherte Grundeinstellungen.
 *
 * Es sind zwei Arten von synchronisierten Speichern definiert:
 * 1. Fr den angemeldeten Benutzer global und lokal;
 * 2. Fr alle Benutzer global und lokal.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 79 $ / $Date: 2013-10-30 17:27:50 +0100 (Mi, 30 Okt 2013) $ / ($Author: LG $)
 */
public class GrundEinstellungen {

    /** Definiert die Art und Ort von Speicher fr den Benutzer. */
    private static final SpeicherArtUndOrt SPEICHER_BENUTZER = ermittleBenutzerSpeicherArtUndOrt();

    /** Definiert die Art und Ort von Speicher fr alle Benutzer. */
    private static final SpeicherArtUndOrt SPEICHER_ALLE_BENUTZER = ermittleAlleBenutzerSpeicherArtUndOrt();

    //~ METHODEN ==============================================================

    /**
     * Ermittle die Instanz der Klasse {@link SpeicherArtUndOrt}}
     * fr alle Benutzer.
     *
     * @return Instanz der Klasse SpeicherArtUndOrt.
     */
    private static SpeicherArtUndOrt ermittleAlleBenutzerSpeicherArtUndOrt() {
        SpeicherFunktionen.ArtOrt[] artOrt = { SpeicherFunktionen.ArtOrt.ALLGEMEIN_DAV, SpeicherFunktionen.ArtOrt.ALLGEMEIN_LOKAL };

        return new SpeicherArtUndOrt(artOrt, true);
    }

    /**
     * Ermittle die Instanz der Klasse {@link SpeicherArtUndOrt}}
     * fr den Benutzer.
     *
     * @return Instanz der Klasse SpeicherArtUndOrt.
     */
    private static SpeicherArtUndOrt ermittleBenutzerSpeicherArtUndOrt() {
        SpeicherFunktionen.ArtOrt[] artOrt = { SpeicherFunktionen.ArtOrt.BENUTZER_DAV, SpeicherFunktionen.ArtOrt.BENUTZER_LOKAL };

        return new SpeicherArtUndOrt(artOrt, true);
    }

    /**
     * Ermittle den Parameterwert fr alle Benutzer als Boolean.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert als Boolean oder false.
     */
    public static boolean ermittleBooleanFuerAlleBenutzer(String parameterID) {
        Boolean b = SpeicherFunktionen.ladeParameterAlsBoolean(parameterID, SPEICHER_ALLE_BENUTZER);

        if (b == null) {
            return false;
        }

        return b;
    }

    /**
     * Ermittle den Parameterwert fr den Benutzer als Boolean.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert als Boolean oder false.
     */
    public static boolean ermittleBooleanFuerBenutzer(String parameterID) {
        Boolean b = SpeicherFunktionen.ladeParameterAlsBoolean(parameterID, SPEICHER_BENUTZER);

        if (b == null) {
            return false;
        }

        return b;
    }

    /**
     * Ermittle den Parameterwert fr alle Benutzer als Integer.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert als Integer oder 0.
     */
    public static int ermittleIntegerFuerAlleBenutzer(String parameterID) {
        Integer i = SpeicherFunktionen.ladeParameterAlsInteger(parameterID, SPEICHER_ALLE_BENUTZER);

        if (i == null) {
            return 0;
        }

        return i;
    }

    /**
     * Ermittle den Parameterwert fr den Benutzer als Integer.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert als Integer oder 0.
     */
    public static int ermittleIntegerFuerBenutzer(String parameterID) {
        Integer i = SpeicherFunktionen.ladeParameterAlsInteger(parameterID, SPEICHER_BENUTZER);

        if (i == null) {
            return 0;
        }

        return i;
    }

    /**
     * Ermittle den Parameterwert fr alle Benutzer als String.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert fr alle Benutzer als String oder null.
     */
    public static String ermittleParameterWertFuerAlleBenutzer(String parameterID) {
        return SpeicherFunktionen.ladeParameter(parameterID, SPEICHER_ALLE_BENUTZER);
    }

    /**
     * Ermittle den Parameterwert fr den Benutzer als String.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return Parameterwert fr den Benutzer als String oder null.
     */
    public static String ermittleParameterWertFuerBenutzer(String parameterID) {
        return SpeicherFunktionen.ladeParameter(parameterID, SPEICHER_BENUTZER);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerAlleBenutzer(String parameterID) {
        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_ALLE_BENUTZER);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param ursache Ursache fr die UrlasserDaten (die werden mit Daten des angemeldeten Benutzer vorbelegt).
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerAlleBenutzer(String parameterID, String ursache) {
        UrlasserInfo urlasserInfo = SpeicherFunktionen.erstelleUrlasserInfo(ursache);

        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_ALLE_BENUTZER, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerAlleBenutzer(String parameterID, UrlasserInfo urlasserInfo) {
        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_ALLE_BENUTZER, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerBenutzer(String parameterID) {
        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_BENUTZER);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param ursache Ursache fr die UrlasserDaten (die werden mit Daten des angemeldeten Benutzer vorbelegt).
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerBenutzer(String parameterID, String ursache) {
        UrlasserInfo urlasserInfo = SpeicherFunktionen.erstelleUrlasserInfo(ursache);

        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_BENUTZER, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer lsche Parameter und sichere alle Speicher.
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' gelscht.
     *
     * @param parameterID ID des zu lschendes Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gelscht wird.
     * false - sonst.
     */
    public static boolean loescheParameterFuerBenutzer(String parameterID, UrlasserInfo urlasserInfo) {
        return SpeicherFunktionen.loescheParameter(parameterID, SPEICHER_BENUTZER, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     * Am Anfang wird der UrlasserDialog aufgemacht.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerAlleBenutzer(String parameterID, String wert) {
        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_ALLE_BENUTZER, wert);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param ursache Ursache fr die UrlasserDaten (die werden mit Daten des angemeldeten Benutzer vorbelegt).
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerAlleBenutzer(String parameterID, String wert, String ursache) {
        UrlasserInfo urlasserInfo = SpeicherFunktionen.erstelleUrlasserInfo(ursache);

        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_ALLE_BENUTZER, wert, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr alle Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerAlleBenutzer(String parameterID, String wert, UrlasserInfo urlasserInfo) {
        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_ALLE_BENUTZER, wert, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     * Am Anfang wird der UrlasserDialog aufgemacht.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerBenutzer(String parameterID, String wert) {
        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_BENUTZER, wert);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param ursache Ursache fr die UrlasserDaten (die werden mit Daten des angemeldeten Benutzer vorbelegt).
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerBenutzer(String parameterID, String wert, String ursache) {
        UrlasserInfo urlasserInfo = SpeicherFunktionen.erstelleUrlasserInfo(ursache);

        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_BENUTZER, wert, urlasserInfo);
    }

    /**
     * Fr alle Speicherarten fr den Benutzer berschreibe Parameter
     * mit dem neuen Wert und sichere alle Speicher
     * (nur falls der Parameter einen anderen Wert hatte).
     * Gleichzeitig wird der Parameter 'AKTUALISIERUNGS_DATUM' berschrieben.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     * Soll in der Klasse SpeicherParameter als 'public final static String' definiert werden.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird.
     * false - sonst.
     */
    public static boolean speichereParameterFuerBenutzer(String parameterID, String wert, UrlasserInfo urlasserInfo) {
        return SpeicherFunktionen.speichereParameter(parameterID, SPEICHER_BENUTZER, wert, urlasserInfo);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 30.10.13
