/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.funktionen.datei;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.SWT;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.OutputStream;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet die statischen DateiFunktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 15 $ / $Date: 2013-09-24 14:21:48 +0200 (Di, 24 Sep 2013) $ / ($Author: LG $)
 */
public class DateiFunktionen {

    /**
     * Ermittle alle Dateien mit dem Namen 'name-ZZZ.typ' aus dem Ordner,
     * wo die Datei fileName liegt, und die Datei fileName selbst,
     * wobei name der Name der angegebenen Datei ist,
     * typ - der Typ  der angegebenen Datei ist und
     * ZZZ - eine Zahl ist.
     *
     * @param fileName Datei.
     *
     * @return Array mit allen nummerierten Dateien des entsprechenden Typs
     * und der Datei selbst.
     */
    public static File[] ermittleAlleNummerierteDatei(String fileName) {
        File         file      = new File(fileName);
        final File   ordner    = file.getParentFile();
        String[]     arNameTyp = ermittleNameUndTypDerDatei(file.getName());
        final String name      = arNameTyp[0];
        final String typ       = arNameTyp[1];

        // ermittle alle Datei:
        // noinspection UnnecessaryLocalVariable
        File[] arAlleDatei = ordner.listFiles(new FilenameFilter() {
            @Override
            public boolean accept(File arg0, String arg1) {

                // Whle alle Datei aus diesem Ordner mit Namen 'name-ZZZ.typ' oder 'name.typ'.
                return ((arg0 == ordner) && ((arg1.matches(name + "-[0-9][0-9][0-9]" + typ)) || (arg1.matches(name + typ))));
            }
        });

        // Falls Array gleich null ist:
        if (arAlleDatei == null) {
            arAlleDatei = new File[0];
        }

        return arAlleDatei;
    }

    /**
     * Ermittle Druckausgabe aus der angegebenen ASCII-Datei.
     *
     * @param dateiName Name der ASCII-Datei.
     *
     * @return Druckausgabe aus der angegebenen ASCII-Datei.
     */
    public static PagePrint ermittleDruckausgabeAusDatei(String dateiName) {
        try {
            LineNumberReader datei = new LineNumberReader(new FileReader(dateiName));
            String           zeile;
            GridPrint        grid = new GridPrint("d");
            TextPrint        text;
            FontData         font = new FontData("Courier New", 12, SWT.NORMAL);

            while ((zeile = datei.readLine()) != null) {
                text = new TextPrint(zeile, font);
                grid.add(text);
            }

            datei.close();

            return new PagePrint(grid);
        }
        catch (FileNotFoundException e) {
            zeigeMeldungZumFileNotFoundException(e, dateiName);
        }
        catch (IOException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Lesen aus der Datei " + dateiName + "!", e));
        }

        return null;
    }

    /**
     * Ermittle den Namen und Typ fr die angegebene Datei.
     *
     * @param fileName Datei, deren Namen und Typ ermittelt werden sollen.
     *
     * @return Array: 1.Element - DateName, 2.Element - TypName (mit dem Punkt vorne).
     */
    public static String[] ermittleNameUndTypDerDatei(String fileName) {
        String[] arNameTyp = new String[2];
        String   name;
        String   typ = "";

        // Index fr Typ der Datei.
        int index = fileName.lastIndexOf(".");

        if (index > -1) {
            name = fileName.substring(0, index);
            typ  = "." + fileName.substring(index + 1);
        } else {
            name = fileName;
        }

        arNameTyp[0] = name;
        arNameTyp[1] = typ;

        return arNameTyp;
    }

    /**
     * Ermittle den Dateinamen, in dem zu der angegebenen Datei
     * die laufende Nummer addiert wird (mit der Bercksichtigung des DateiTyps).
     * Falls laufNummer gleich 0 ist, wird fileName zurckgegeben.
     *
     * @param fileName Datei,zu der die laufende Nummer addiert wird.
     * @param laufNummer Laufende Nummer.
     *
     * @return Dateinamen mit der laufenden Nummer (dreistellig mit 0 am Anfang),
     * oder fileName, falls laufNummer gleich 0 ist.
     */
    public static String ermittleNummeriertenDateinamen(String fileName, int laufNummer) {
        if (laufNummer == 0) {
            return fileName;
        } else {
            String[] arNameTyp = ermittleNameUndTypDerDatei(fileName);
            String   name      = arNameTyp[0];
            String   typ       = arNameTyp[1];

            return name + "-" + String.format("%03d", laufNummer) + typ;
        }
    }

    /**
     * Ermittle den Pfad und den Dateinamen von im Parameter bergebenem Dateinamen.
     *
     * @param pathFile DateiName und Pfad.
     *
     * @return Das aus zwei Strings bestehende Array.
     * Erstes Element - der Pfad, zweites - der Filename von im Parameter bergebenem Dateinamen.
     */
    public static String[] ermittlePfadUndDateinamen(String pathFile) {
        int    index    = pathFile.lastIndexOf(File.separator);
        String fileName = "";
        String pathName = "";

        if (index > -1) {
            fileName = pathFile.substring(index + 1);
            pathName = pathFile.substring(0, index);
        }

        return new String[] { pathName, fileName };
    }

    /**
     * Ersetze den Text textAlt durch den Text textNeu berall in der Datei.
     *
     * @param fileName Name der Datei.
     * @param textAlt Text, der ersetzt wird.
     * @param textNeu Neuer Text.
     *
     * @return true, falls der Text berall in der Datei erfolgreich ersetzt wurde,
     * das aufgetretene Exception sonst.
     */
    public static Exception erzestzeTextInDatei(String fileName, String textAlt, String textNeu) {

        // Der neuer Inhalt wird in die temp. Datei geschrieben
        // und dann wird diese Datei in fileName umbenannt.
        File file = new File(fileName);

        // File fileTemp=new File(fileName+".tmp");
        File fileTemp = erzeugeTempFile(fileName);

        // Erzeuge TempFile fr Output:
        FileOutputStream out;

        try {
            out = new FileOutputStream(fileTemp);
        }
        catch (FileNotFoundException e) {
            zeigeMeldungZumFileNotFoundException(e, fileTemp.toString());

            return e;
        }

        // Kopiere den Inhalt von XMLFile in TempFile,
        // dabei ersetze den Text:
        try {
            LineNumberReader in = new LineNumberReader(new FileReader(file));
            String           line;
            boolean          start = true;

            while ((line = in.readLine()) != null) {
                if (start) {
                    start = false;
                } else {
                    out.write(NEWLINE.getBytes());
                }

                line = line.replace(textAlt, textNeu);
                out.write(line.getBytes());
            }

            out.close();
            in.close();
        }
        catch (FileNotFoundException e) {
            zeigeMeldungZumFileNotFoundException(e, fileName);

            return e;
        }
        catch (IOException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Lesen aus der Datei " + fileName + " bsw. beim Schreiben in die Datei " + fileTemp + "!", e));

            return e;
        }

        // fr testZwecke:
        // vergleicheFile(fileName, fileName+".tmp");
        // Umbenenne TempFile in FileName:
        // noinspection ResultOfMethodCallIgnored
        file.delete();

        if (!fileTemp.renameTo(file)) {
            String fehler = "Die Datei " + fileTemp + " kann nicht umbenennt werden!";

            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, fehler));

            return new Exception(fehler);
        }

        return null;
    }

    /**
     * Erzeuge eine temporre Datei, die noch nicht existiert,
     * mit dem Namen fileName+i+".tmp".
     *
     * @param fileName Fr die Namenbildung.
     * @return eine temporre Datei, die noch nicht existiert,
     * mit dem Namen fileName+i+".tmp".
     */
    public static File erzeugeTempFile(String fileName) {
        File          fileTemp;
        int           i = 0;
        StringBuilder fileNameTemp;

        do {
            fileNameTemp = new StringBuilder(fileName).append(i).append(".tmp");
            fileTemp     = new File(fileNameTemp.toString());
            i++;
        } while (fileTemp.exists());

        return fileTemp;
    }

    /**
     * Kopiere File.
     *
     * @param quelleFile Quelldatei.
     * @param zielFile Zieldatei.
     *
     * @return null, falls das Kopieren erfolgreich war,
     * das aufgetretene Exception sonst.
     */
    public static Exception kopiereDatei(String quelleFile, String zielFile) {
        int    buffersize = 16348;
        byte[] buffer     = new byte[buffersize];

        try {
            InputStream  in  = new FileInputStream(quelleFile);
            OutputStream out = new FileOutputStream(zielFile);
            int          bytes;

            while ((bytes = in.read(buffer)) > 0) {
                out.write(buffer, 0, bytes);
            }

            in.close();
            out.close();
        }
        catch (FileNotFoundException e) {
            zeigeMeldungZumFileNotFoundException(e, quelleFile);

            return e;
        }
        catch (IOException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Lesen aus der Datei " + quelleFile + " bsw. beim Schreiben in die Datei " + zielFile + "!", e));

            return e;
        }

        return null;
    }

    /**
     * TestZwecke: vergleiche den Inhalt von zwei Dateien
     * und den Unterschied zeige in der Konsole.
     * @param f1 Name der Datei1.
     * @param f2 Name der Datei2.
     */
    public static void vergleicheDateien(String f1, String f2) {
        try {
            LineNumberReader in1 = new LineNumberReader(new FileReader(f1));
            LineNumberReader in2 = new LineNumberReader(new FileReader(f2));
            String           line1;
            String           line2;

            while ((line1 = in1.readLine()) != null) {
                line2 = in2.readLine();

                if (!line1.equals(line2)) {
                    System.out.println("------");
                    System.out.println(line1);
                    System.out.println(line2);
                    System.out.println("------");
                } else {

                    // System.out.println("ok");
                }
            }

            in1.close();
            in2.close();
        }
        catch (FileNotFoundException e) {
            zeigeMeldungZumFileNotFoundException(e, f1 + " bzw. " + f2);
        }
        catch (IOException e) {
            e.printStackTrace();
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Lesen aus der Datei " + f1 + " oder " + f2 + "!", e));
        }
    }

    /**
     * Zeige Standard-Fehlermeldung beim FileNotFoundException.
     *
     * @param e FileNotFoundException.
     * @param dateiName DateiName.
     */
    private static void zeigeMeldungZumFileNotFoundException(FileNotFoundException e, String dateiName) {
        Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Datei <" + dateiName + "> kann nicht gefunden werden!", e));
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 24.09.13
