/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.gui.pruefung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.PlatformUI;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DecimalFormat;
import java.text.NumberFormat;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die abstrakte Klasse dient fr die Prfung jedes Felds im Dialog beim Verlassen des FeldFokus.
 *
 * Fr bestimmten Dialog soll man eine Klasse definieren, die diese Klasse erbt.
 * In der Klasse, wo Dialog definiert ist, soll der Aufruf der Methode
 * addFocusListener(...) fr alle Felder, die man berprfen will, implementiert werden.
 * Dazu kann man die Methode aufrufAddFocusListener() aus dieser Klasse benutzen
 * oder ganz normal ohne Reflection den Listener aufrufen
 * (z.B. cbtnOPXProjekt.addFocusListener(cbtnOPXProjekt, new Klasse...("cbtnOPXProjekt", this))).
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
abstract public class PruefungDialogFelder extends FocusAdapter {

    /**
     * Zeigt, ob die EingabeFeldPrfung fr den Feld stattfinden soll.
     * Wird auf <tt>false</tt> gesetzt, falls eine Prfung fr irgendwelchen Feld angefangen hat
     * und noch nicht abgeschlossen ist.
     */
    static protected boolean _isPruefen = true;

    /** Lokales Dezimalformat. */
    public static final DecimalFormat DECIMAL_FORMAT;

    /** Dezimales Separator. */
    public static final String DECIMAL_SEPARATOR;

    //~ STATISCHE INITIALISIERER ==============================================

    static {

        // Definiere das lokale Format und da definiertes DecimalSeparator:
        NumberFormat nf = NumberFormat.getNumberInstance();

        if (nf instanceof DecimalFormat) {
            DECIMAL_FORMAT = (DecimalFormat) nf;
        } else {
            DECIMAL_FORMAT = new DecimalFormat();
        }

        DECIMAL_SEPARATOR = "" + DECIMAL_FORMAT.getDecimalFormatSymbols().getDecimalSeparator();
    }

    //~ FELDER ================================================================

    /**
     * Instanz der Klasse, wo die Felder definiert sind.
     */
    protected Control _guiInstanz;

    /**
     * Name der Instanzvariable, die den Feld deklariert.
     */
    protected String _variableName;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse.
     *
     * @param variableName Initialisiert die Eigenschaft '_variableName'.
     * @param guiInstanz Initialisiert die Eigenschaft '_guiInstanz'.
     */
    protected PruefungDialogFelder(String variableName, Control guiInstanz) {
        super();

        // Entferne '_' vom Variablennamen, falls vorhanden:
        if (variableName.subSequence(0, 1).equals("_")) {
            _variableName = variableName.substring(1);
        } else {
            _variableName = variableName;
        }

        _guiInstanz = guiInstanz;
        _isPruefen  = true;
    }

    //~ METHODEN ==============================================================

    /**
     * Rufe die Methode addFocusListener() fr das Objekt feldInstanz (falls das nicht null ist) auf.
     *
     * @param feldInstanz Objekt (InstanzVariable), auf das der Aufruf von addFocusListener()
     * angewandt wird.
     * Dieses Objekt soll Instanz der Klasse "org.eclipse.swt.widgets.Control" sein.
     * @param focusListenerInstanz Instanz der GUIKlasse, wo diese abstrakte Klasse implementiert wird.
     */
    static public void aufrufAddFocusListener(Object feldInstanz, FocusListener focusListenerInstanz) {
        if (feldInstanz != null) {
            try {

                // in Klasse Control suche Method addFocusListener():
                Class<?> klasse              = Class.forName("org.eclipse.swt.widgets.Control");
                Class<?> klasseFocusListener = Class.forName("org.eclipse.swt.events.FocusListener");
                Method   addFocusFunktion    = klasse.getMethod("addFocusListener", new Class[] { klasseFocusListener });

                // Aufruf der Methode addFocusListener:
                addFocusFunktion.invoke(feldInstanz, focusListenerInstanz);
            }
            catch (ClassNotFoundException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (SecurityException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (NoSuchMethodException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (IllegalArgumentException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (IllegalAccessException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (InvocationTargetException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }
    }

    /**
     * Ruft die getFunktion mit dem Namen "getVariablenName"
     * aus der Instanz klasseInstanz auf.
     * (der 1.Parameter wird umgewandelt so, dass 1.Buchstabe gross geschrieben wird,
     * und falls erster Buchstabe _ ist, wird die weggelassen)
     *
     * @param variableName Name der Variable.
     * @param klasseInstanz Instanz der Klasse, aus der die getFunktion aufgerufen wird.
     *
     * @return Ergebnis von der getFunktion.
     */
    static public Object aufrufGetFunktion(String variableName, Object klasseInstanz) {

        //
        String variable = variableName;

        if (variableName.subSequence(0, 1).equals("_")) {
            variable = variable.substring(1);
        }

        // ndere den Namen variableName, so, dass 1.Buchstabe gross geschrieben wird:
        variable = variable.substring(0, 1).toUpperCase() + variable.substring(1);

        Method getFunktion;
        Object ergebnisGetFunktion = null;

        // Suche getFunktion und rufe die auf:
        try {
            getFunktion         = klasseInstanz.getClass().getDeclaredMethod("get" + variable, new Class[0]);
            ergebnisGetFunktion = getFunktion.invoke(klasseInstanz);
        }
        catch (SecurityException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (NoSuchMethodException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (IllegalArgumentException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (IllegalAccessException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (InvocationTargetException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        return ergebnisGetFunktion;
    }

    /*
     * @see org.eclipse.swt.events.FocusAdapter#focusLost(org.eclipse.swt.events.FocusEvent)
     */
    public void focusLost(FocusEvent evt) {
        if (_isPruefen) {
            _isPruefen = false;
            pruefeDialogFelder(evt);
        }
    }

    /**
     * Prft, ob der text eine positive Zahl ist (als Trennzeichen kann man [,.;] benutzen).
     * <p>
     * Falls  der 2.Parameter 'vorkommaZahl' grer als 0 ist,
     * wird die Anzahl der Ziffern im text vor dem Trennzeichen kontrolliert
     * (falls ntig wird vorne mit 0 belegt,
     * oder falls im text die Anzahl grer ist,
     * wird ein leeres String zurckgegeben).
     * </p>
     * Falls  der 3.Parameter 'nachkommaZahl' grer als 0 ist,
     * wird die Anzahl der Ziffern im text nach dem Trennzeichen kontrolliert
     * (falls ntig wird hinten mit 0 belegt,
     * oder falls im text die Anzahl grer ist,
     * wird das aufgerundet).
     * <p>
     * Falls text keine positive Zahl darstellt, wird ein leeres String zurckgegeben.
     * </p>
     *
     * @param text Text, der berprft werden soll.
     * @param vorkommaZahl Format: Anzahl der Ziffern vor Trennzeichen oder 0, falls nicht formatiert werden soll.
     * @param nachkommaZahl Format: Anzahl der Ziffern nach Trennzeichen oder 0.
     *
     * @return Der formatierte Text entsprechend Parameter 2 und 3 (falls diese Parameter jeweils grer als 0 sind) String
     * oder leeres String, falls text keine Zahl darstellt, oder sich nicht die vorkommaZahl anpassen lsst.
     */
    static public String formatiereTextAlsPositiveZahl(String text, int vorkommaZahl, int nachkommaZahl) {
        return formatiereTextAlsZahl(text, vorkommaZahl, nachkommaZahl, false);
    }

    /**
     * Prft, ob der text eine Zahl ist (als Trennzeichen kann man [,.;] benutzen).
     * <p>
     * Falls  der 2.Parameter 'vorkommaZahl' grer als 0 ist,
     * wird die Anzahl der Ziffern im text vor dem Trennzeichen kontrolliert
     * (falls ntig wird vorne mit 0 belegt,
     * oder falls im text die Anzahl grer ist,
     * wird ein leeres String zurckgegeben).
     * </p>
     * Falls  der 3.Parameter 'nachkommaZahl' grer als 0 ist,
     * wird die Anzahl der Ziffern im text nach dem Trennzeichen kontrolliert
     * (falls ntig wird hinten mit 0 belegt,
     * oder falls im text die Anzahl grer ist,
     * wird das aufgerundet).
     * <p>
     * Falls text keine Zahl darstellt, wird ein leeres String zurckgegeben.
     * </p>
     *
     * @param text Text, der berprft werden soll.
     * @param vorkommaZahl Format: Anzahl der Ziffern vor Trennzeichen oder 0, falls nicht formatiert werden soll.
     * @param nachkommaZahl Format: Anzahl der Ziffern nach Trennzeichen oder 0.
     * @param darfNegativeSein Stellt fest, ob die Zahl negativ werden darf.
     *
     * @return Der formatierte Text entsprechend Parameter 2 und 3 (falls diese Parameter jeweils grer als 0 sind) String
     * oder leeres String, falls text keine Zahl darstellt, oder sich nicht die vorkommaZahl anpassen lsst.
     */
    static public String formatiereTextAlsZahl(String text, int vorkommaZahl, int nachkommaZahl, boolean darfNegativeSein) {
        String formText = "";
        String negativ  = "";

        text = text.trim();

        // Prfe erstes Zeichen:
        if (text.length() > 0) {

            // Lsche vorne '+', falls vorhanden:
            if (text.substring(0, 1).equals("+")) {
                text = text.substring(1);
            }

            // Lsche vorne '-', falls vorhanden und merke das Zeichen,
            // falls die zu formatierte Zahl negative werden darf:
            if (text.substring(0, 1).equals("-")) {
                text = text.substring(1);

                if (darfNegativeSein) {

                    // Merke die negative Zahl:
                    negativ = "-";
                }
            }
        }

        // berprfe, ob text eine Zahl darstellt:
        if (text.matches("([0-9]*)([.,;]?)([0-9]*)")) {
            formText = text;

            int index = formText.indexOf(",");

            if (index < 0) {
                index = formText.indexOf(";");
            }

            // berprfe, ob die Gre der zu formatierte text bis zum Trennzeichen
            // nicht die angegebene vorkommaZahl berschreitet:
            if ((vorkommaZahl > 0) && ((index > vorkommaZahl) || ((index < 0) && (formText.length() > vorkommaZahl)))) {
                formText = "";
            } else {

                // Formatiere Text so, dass der als Trennzeichen einen Punkt hat:
                if (index > -1) {
                    formText = formText.substring(0, index) + "." + formText.substring(index + 1);
                }

                // Falls text leer ist, mache darauf 0:
                if (formText.length() == 0) {
                    formText = "0";
                }

                formText = konvertiereDoubleInText(Double.valueOf(formText), vorkommaZahl, nachkommaZahl);
                formText = negativ + formText;
            }
        }

        return formText;
    }

    /**
     * Konvertiere Text in double (bis 3 Nachkommastellen gerundet).
     *
     * @param textFeld GUI-Element von Typ Text, deren Text in Double umgewandelt wird.
     * @param feldName Name des Text-Feldes. Wird fr die Fehlermeldung benutzt.
     * @param zeigenFehlerMeldung Stellt fest, ob die Fehlermeldung angezeigt werden soll.
     *
     * @return Aus dem Text konvertierte Double-Zahl.
     */
    public static double holeDouble(Text textFeld, String feldName, boolean zeigenFehlerMeldung) {
        double dZahl = 0;

        try {
            dZahl = PruefungDialogFelder.konvertiereTextInDouble(textFeld.getText());

            // Runde double bis 3 Nachkommastelle:
            dZahl = Math.round(dZahl * 1000.) / 1000.;
        }
        catch (NumberFormatException e) {
            if (zeigenFehlerMeldung) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler im GUI-Element: " + feldName + " muss eine Zahl sein!", e));
            }
        }

        return dZahl;
    }

    /**
     * Konvertiere Text in int.
     *
     * @param textFeld GUI-Element von Typ Text, deren Text in int umgewandelt wird.
     * @param feldName Name des Text-Feldes. Wird fr die Fehlermeldung benutzt.
     * @param zeigenFehlerMeldung Stellt fest, ob die Fehlermeldung angezeigt werden soll.
     *
     * @return Aus dem Text konvertierte int-Zahl.
     */
    public static int holeInt(Text textFeld, String feldName, boolean zeigenFehlerMeldung) {
        int iZahl = 0;

        try {
            iZahl = PruefungDialogFelder.konvertiereTextInInt(textFeld.getText());
        }
        catch (NumberFormatException e) {
            if (zeigenFehlerMeldung) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler im GUI-Element: " + feldName + " muss eine Zahl sein!", e));
            }
        }

        return iZahl;
    }

    /**
     * Konvertiere Text in long.
     *
     * @param textFeld GUI-Element von Typ Text, deren Text in Long umgewandelt wird.
     * @param feldName Name des Text-Feldes. Wird fr die Fehlermeldung benutzt.
     * @param zeigenFehlerMeldung Stellt fest, ob die Fehlermeldung angezeigt werden soll.
     *
     * @return Aus dem Text konvertierte Long-Zahl.
     */
    public static long holeLong(Text textFeld, String feldName, boolean zeigenFehlerMeldung) {
        long lZahl = 0;

        try {
            lZahl = PruefungDialogFelder.konvertiereTextInLong(textFeld.getText());
        }
        catch (NumberFormatException e) {
            if (zeigenFehlerMeldung) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler im GUI-Element: " + feldName + " muss eine Zahl sein!", e));
            }
        }

        return lZahl;
    }

    //~ GET METHODEN ==========================================================

    /**
     * berprft, ob angegebener Text eine EMail darstellt.
     * Leeres String ist erlaubt.
     *
     * @param text der zu berprfende Text.
     *
     * @return <tt>true</tt> falls der Text eine EMail darstellt oder ein leeres String ist, <tt>false</tt> sonst.
     */
    static public boolean isEmail(String text) {
        boolean ergebnis = true;

        if (!text.equals("")) {
            String regAusdruck = "\\w[-.\\w]*\\@[-a-zA-Z0-9]+(\\.[-a-zA-Z0-9]+)*\\.(com|edu|gov|int|mil|net|org|biz|info|name|museum|coop|aero|[a-z][a-z])";

            ergebnis = text.matches(regAusdruck);
        }

        return ergebnis;
    }

    /**
     * Getter fr Eigenschaft '_isPruefen'.
     *
     * @return isPruefen Eigenschaft '_isPruefen'.
     *
     * @see #_isPruefen
     */
    public static boolean isPruefen() {
        return _isPruefen;
    }

    /**
     * Prft, ob der Text eine positive intZahl ist und, falls der zweite Parameter grer als 0 ist,
     * ob die Lnger des Texts gleich 'groesse' ist.
     *
     * @param text der zu berprfende Text.
     * @param groesse gibt die ntige Gre der Text. Falls gleich 0 ist, wird die Gre nicht berprft.
     *
     * @return <tt>true</tt> falls der Text eine Zahl ist und, falls der zweite Parameter grer als 0 ist,
     * die richtige Gre hat.
     */
    static public boolean isZahl(String text, int groesse) {
        boolean ergebnis = false;

        if (text.matches("[0-9]+")) {
            ergebnis = true;
        }

        if ((groesse > 0) && (text.length() != groesse)) {
            ergebnis = false;
        }

        return ergebnis;
    }

    //~ METHODEN ==============================================================

    /**
     * Konvertiere Double in einen Text.
     *
     * @param zahl Die zu konvertierte Zahl.
     * @param nachkommaZahl Anzahl der Ziffern nach Trennzeichen.
     *
     * @return Text-Representation der Double-Zahl.
     */
    static public String konvertiereDoubleInText(double zahl, int nachkommaZahl) {
        return konvertiereDoubleInText(zahl, 0, nachkommaZahl);
    }

    /**
     * Konvertiere Double in einen Text.
     *
     * @param zahl Die zu konvertierte Zahl.
     * @param vorkommaZahl Format: Anzahl der Ziffern vor Trennzeichen oder 0, falls nicht formatiert werden soll.
     * @param nachkommaZahl Anzahl der Ziffern nach Trennzeichen.
     *
     * @return Text-Representation der Double-Zahl.
     */
    static public String konvertiereDoubleInText(double zahl, int vorkommaZahl, int nachkommaZahl) {

        // Definiere FormatString fr den VorkommaTeil:
        StringBuilder format;

        if (vorkommaZahl > 0) {
            format = new StringBuilder("");

            for (int i = 0; i < vorkommaZahl; i++) {
                format.append("0");
            }
        } else {
            format = new StringBuilder("#0");
        }

        // Definiere FormatString fr den NachkommaTeil:
        if (nachkommaZahl > 0) {
            format.append(DECIMAL_SEPARATOR);

            for (int i = 0; i < nachkommaZahl; i++) {
                format.append("0");
            }
        }

        // Formatiere Zahl
        DECIMAL_FORMAT.applyLocalizedPattern(format.toString());

        return DECIMAL_FORMAT.format(zahl);
    }

    /**
     * Konvertiere einen Text in Double.
     * Falls der Text DecimalSeparator enthlt, ersetze das erste durch '.'.
     * Falls das Ergebnis nicht konvertiert werden kann, tritt NumberFormatException auf.
     *
     * @param text Der zu konvertierte Text.
     *
     * @return Eine DoubleZahl, falls mit Erfolg konvertiert wird.
     *
     * @throws NumberFormatException, falls der Text nicht in Double konvertiert werden kann.
     */
    static public double konvertiereTextInDouble(String text) throws NumberFormatException {

        // Ersetze DecimalSeparator durch '.':
        text = text.replace(DECIMAL_SEPARATOR, ".");

        // Konvertiere:
        return Double.parseDouble(text);
    }

    /**
     * Konvertiert einen Text in int.
     * Falls der Text DecimalSeparator enthlt, ersetze das erste durch '.'.
     * Falls das Ergebnis nicht konvertiert werden kann, tritt NumberFormatException auf.
     *
     * @param text Der zu konvertierte Text.
     *
     * @return Eine IntZahl, falls mit Erfolg konvertiert wird.
     *
     * @throws NumberFormatException, falls der Text nicht in int konvertiert werden kann.
     */
    static public int konvertiereTextInInt(String text) throws NumberFormatException {

        // Ersetze DecimalSeparator durch '.':
        text = text.replace(DECIMAL_SEPARATOR, ".");

        // Konvertiere:
        return Integer.parseInt(text);
    }

    /**
     * Konvertiert einen Text in Long.
     * Falls der Text DecimalSeparator enthlt, ersetze das erste durch '.'.
     * Falls das Ergebnis nicht konvertiert werden kann, tritt NumberFormatException auf.
     *
     * @param text Der zu konvertierte Text.
     *
     * @return Eine LongZahl, falls mit Erfolg konvertiert wird.
     *
     * @throws NumberFormatException, falls der Text nicht in Long konvertiert werden kann.
     */
    static public long konvertiereTextInLong(String text) throws NumberFormatException {

        // Ersetze DecimalSeparator durch '.':
        text = text.replace(DECIMAL_SEPARATOR, ".");

        // Konvertiere:
        return Long.parseLong(text);
    }

    /**
     * Ruft die zum Feld zugehrige PrfFunktion auf, falls die existiert,
     * zeigt den MeldungDialog, falls ntig ist
     * und falls die FeldEingabe nicht korrekt ist,
     * erlaubt nicht den Fokus vom Feld zu verlassen.
     *
     * @param evt FocusEvent.
     */
    protected void pruefeDialogFelder(FocusEvent evt) {
        final FocusEvent event = evt;

        // StandartInizialisierung von PrfFunktionErgebnis. Wird auch benutzt,
        // wenn keine PrfFunktion gefunden wird.
        boolean isEingabeRichtig = true;
        boolean isMeldung        = false;
        int     meldungTyp       = 0;
        String  warnungText      = "";

        // Suche die PrfFunktion fr den Feld und rufe sie auf, falls gefunden.
        try {
            Class<?> klasse                = this.getClass();
            Method   pruefFunktion         = klasse.getDeclaredMethod("pruefeFeld_" + _variableName, new Class[0]);
            Object   ergebnisPruefFunktion = pruefFunktion.invoke(this);

            if (ergebnisPruefFunktion instanceof PruefungErgebnis) {
                PruefungErgebnis pruefungErgebnis = (PruefungErgebnis) ergebnisPruefFunktion;

                isEingabeRichtig = pruefungErgebnis._isEingabeRichtig;
                isMeldung        = pruefungErgebnis._isMeldung;
                meldungTyp       = pruefungErgebnis._meldungTyp;
                warnungText      = pruefungErgebnis._meldungText;
            }
        }
        catch (SecurityException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (NoSuchMethodException e) {

            // Diese Exception kann oft passieren. Ist keine _Fehler,
            // weil nicht alle Felder eine PrfFunktion haben sollen.
            // Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (IllegalArgumentException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (IllegalAccessException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
        catch (InvocationTargetException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        // Falls die Variable isMeldung auf true gesetzt wird, zeige den MeldungDialog.
        // Der Typ des Dialogs hngt von Variable meldungTyp (1-Info,2-Warnung,3-_Fehler)ab.
        if (isMeldung) {
            Shell aktShell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

            if (meldungTyp == 1) {
                MessageDialog.openInformation(aktShell, "Information", warnungText);
            } else if (meldungTyp == 2) {
                MessageDialog.openWarning(aktShell, "Warnung", warnungText);
            } else if (meldungTyp == 3) {
                MessageDialog.openError(aktShell, "Fehler", warnungText);
            }
        }

        _guiInstanz.setFocus();

        // Falls die Eingabe im Feld nicht korrekt ist,
        // setzte das Fokus auf dem Feld.
        if (!isEingabeRichtig) {
            Display.getCurrent().asyncExec(new Runnable() {
                @Override
                public void run() {
                    Object source = event.getSource();

                    try {
                        Method setFocusFunktion = source.getClass().getMethod("forceFocus", new Class[0]);

                        setFocusFunktion.invoke(source);
                    }
                    catch (SecurityException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                    catch (NoSuchMethodException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                    catch (IllegalArgumentException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                    catch (IllegalAccessException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                    catch (InvocationTargetException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }

                    _isPruefen = true;
                }
            });
        } else {
            _isPruefen = true;
        }
    }

    /**
     * MusterFunktion fr alle PrfFunktionen.
     * FunktionName besteht aus "pruefeFeld_" und den Namen der InstanzVariable, die den Feld deklariert.
     * Soll public sein.
     * RckgabeWert soll von Typ PruefungErgebnis sein.
     * Alle Elemente von Array werden in einer Funktion bearbeiten.
     *
     * Prfe die FeldEingabe auf die Korrektheit.
     *
     * @return <tt>true</tt> fr die Instanzvariable _isEingabeRichtig, falls die Eingabe korrekt ist, <tt>false</tt> sonst;
     * <tt>true</tt> fr die Instanzvariable _isMeldung, falls die MeldungDialog angezeigt werden soll, <tt>false</tt> sonst;
     * den Typ der Meldung (1-Info, 2-Warnung, 3-_Fehler);
     * den Text in der Instanzvariable _meldungText fr den MeldungDialog.
     */
    @SuppressWarnings({ "UnusedDeclaration" })
    public PruefungErgebnis pruefeFeld_XXX() {
        return new PruefungErgebnis();
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * Die Klasse beschreibt das Ergebnis von PrfFunktionen fr jeden Feld.
     */
    protected class PruefungErgebnis {

        /**
         * Ermittelt, ob die Eingabe im Feld korrekt ist.
         */
        private boolean _isEingabeRichtig;

        /**
         * Ermittelt, ob die Meldung erzeugt werden soll.
         */
        private boolean _isMeldung;

        /**
         * Der Text fr den MeldungDialog, der aufgemacht wird,
         * falls die Variable _isMeldung auf true gesetzt wird.
         */
        private String _meldungText;

        /**
         * Definiert den Typ der Meldung.
         * 1-Info, 2-Warnung, 3-_Fehler.
         */
        private int _meldungTyp;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        /**
         * Erzeugt eine Instanz der Klasse mit DefaultWerten (Eingabe ist richtig, keine Meldung).
         */
        public PruefungErgebnis() {
            _isEingabeRichtig = true;
            _isMeldung        = false;
            _meldungTyp       = 0;
            _meldungText      = "";
        }

        //~ GET METHODEN ======================================================

        /**
         * Getter fr Eigenschaft 'isEingabeRichtig'.
         *
         * @return isEingabeRichtig Wert der Eigenschaft 'isEingabeRichtig'.
         *
         * @see #_isEingabeRichtig
         */
        public boolean isEingabeRichtig() {
            return _isEingabeRichtig;
        }

        //~ SET METHODEN ======================================================

        /**
         * Setter fr Eigenschaft 'isEingabeRichtig'.
         *
         * @param isEingabeRichtig Wert der Eigenschaft 'isEingabeRichtig'.
         *
         * @see #_isEingabeRichtig
         */
        public void setIsEingabeRichtig(boolean isEingabeRichtig) {
            _isEingabeRichtig = isEingabeRichtig;
        }

        /**
         * Setter fr Eigenschaft 'isMeldung'.
         *
         * @param isMeldung Wert der Eigenschaft 'isMeldung'.
         *
         * @see #_isMeldung
         */
        public void setIsMeldung(boolean isMeldung) {
            _isMeldung = isMeldung;
        }

        /**
         * Setter fr Eigenschaft 'meldungText'.
         *
         * @param meldungText Wert der Eigenschaft 'meldungText'.
         *
         * @see #_meldungText
         */
        public void setMeldungText(String meldungText) {
            _meldungText = meldungText;
        }

        /**
         * Setter fr Eigenschaft 'meldungTyp'.
         * 1-Info, 2-Warnung, 3-_Fehler.
         *
         * @param meldungTyp Wert der Eigenschaft 'meldungTyp'.
         *
         * @see #_meldungTyp
         */
        public void setMeldungTyp(int meldungTyp) {
            _meldungTyp = meldungTyp;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.02.11
