/*
 * @(#)DatenVewMq.java   21.05.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.mqnavigation.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

import de.inovat.buv.gtm.datvew.lib.AuswerteQuerschnitt;
import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.Messquerschnitt;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.mqnavigation.Activator;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.Status;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Datenverwaltung ist als Singleton realisiert.
 * Initialisiert die MQ-Daten und erlaubt den Zugriff auf Daten.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 380 $ / $Date: 2013-05-22 15:51:27 +0200 (Mi, 22 Mai 2013) $ / ($Author: LG $)
 */
public class DatenVewMq {
    private static final String TYP_MQ_ALLGEMEIN = "typ.messQuerschnittAllgemein";

    /** Einzige Instanz der Klasse. */
    private static DatenVewMq _instanz = new DatenVewMq();

    /** Die Datenverteiler-Konfiguration. */
    private static DataModel _davKonf;

    //~ FELDER ================================================================

    /**
     * Stellt fest, ob die Initialisierung der Gerte abgeschlossen ist (s.{@link #DatenVewMq()}).
     *
     * WICHTIG:
     * Die get-Methoden  MSSEN auf das Ende der Initialisierung warten.
     */
    private boolean _istInitialisierungAbgeschlossen = false;

    /**
     * Menge enthlt alle MQs und DaV-Typ, zu dem der MQ gehrt.
     *
     * Key: MQ.
     * Value Typ des MQ.
     */
    private Map<Messquerschnitt, DavSystemObjekt> _mapMqTyp;

    /**
     * Menge enthlt alle DaV-Typen, die von Typ TYP_MQ_ALLGEMEIN abgeleitet sind
     * und die Liste der zugehrigen Elementen.
     *
     * Key: MQ-Typ.
     * Value: Nach PID sortierte Liste alle MQs vom angegebenen Typ.
     */
    private Map<DavSystemObjekt, List<Messquerschnitt>> _mapTypMq;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse DatenVewMq.
     */
    private DatenVewMq() {

        //
        _davKonf = DavDatenVew.getInstanz().getDav().getDataModel();

        // -------------------------------------------------------------------
        // WICHTIG
        // Die Initialisierung der Objekte luft langsam ===>
        // wird in JOB ausgelagert.
        // ===>
        // Die get-Methoden MSSEN auf das Ende der Initialisierung warten.
        Job job = new Job("Initialisiere MQ...") {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                initMq();

                //
                _istInitialisierungAbgeschlossen = true;

                return Status.OK_STATUS;
            }
        };

        job.schedule();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittelt einzige Instanz der Klasse DatenVewMq.
     *
     * @return Einzige Instanz der Klasse DatenVewMq.
     */
    public static DatenVewMq getInstanz() {
        return _instanz;
    }

    /**
     * Ermittle sortierte Liste alle MQs.
     *
     * @return Nach PID sortierte Liste alle MQs.
     */
    public List<Messquerschnitt> getListeAlleMqSortiert() {
        pauseBisEndeDerInitialisierung();

        //
        List<Messquerschnitt> listeMq = new ArrayList<Messquerschnitt>();

        for (DavSystemObjekt typ : _mapTypMq.keySet()) {
            listeMq.addAll(_mapTypMq.get(typ));
        }

        Collections.sort(listeMq, DatenVew.COMPARATOR_SO_PID);

        return listeMq;
    }

    /**
     * Ermittle sortierte Liste alle MQ-Typs.
     *
     * @return Nach PID sortierte Liste alle MQ-Typs.
     */
    public List<DavSystemObjekt> getListeAlleMqTypsSortiert() {
        pauseBisEndeDerInitialisierung();

        //
        List<DavSystemObjekt> listeMqTyp = new ArrayList<DavSystemObjekt>();

        listeMqTyp.addAll(_mapTypMq.keySet());
        Collections.sort(listeMqTyp, DatenVew.COMPARATOR_SO_PID);

        return listeMqTyp;
    }

    /**
     * Ermittle sortierte Liste alle MQs des angegebenen Typs.
     *
     * @param mqTyp TypMessQuerschnittAllgemein.
     *
     * @return Nach PID sortierte Liste alle MQs des angegebenen Typs.
     */
    public List<Messquerschnitt> getListeAlleMqVomTypSortiert(DavSystemObjekt mqTyp) {
        pauseBisEndeDerInitialisierung();

        //
        return _mapTypMq.get(mqTyp);
    }

    /**
     * Ermittle den Typ des angegebenes MQ.
     *
     * @param mq Messquerschnitt.
     *
     * @return Typ des angegebenes MQ.
     */
    public DavSystemObjekt getTypVonMq(Messquerschnitt mq) {
        pauseBisEndeDerInitialisierung();

        //
        return _mapMqTyp.get(mq);
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere alle Objekte.
     */
    private void initMq() {

        // -----------------------------------------------------------
        // Hole alle MQ von Zhlstellen ab:
        Map<String, Messquerschnitt> mapMqZs = new HashMap<String, Messquerschnitt>();

        for (ZaehlStelle zs : DatenVew.getInstanz().getListeZaehlStellen()) {
            for (AuswerteQuerschnitt awq : zs.getArAWQ()) {
                mapMqZs.put(awq.getMessquerschnitt().getSystemObjekt().getPidOrNameOrId(), awq.getMessquerschnitt());
            }
        }

        // -----------------------------------------------------------
        // Initialisiere alle Typ-Objekte TypMessQuerschnittAllgemein mit zugehrigen MQs:
        _mapTypMq = new HashMap<DavSystemObjekt, List<Messquerschnitt>>();
        _mapMqTyp = new HashMap<Messquerschnitt, DavSystemObjekt>();

        Map<String, DavSystemObjekt> mapTypMq = new HashMap<String, DavSystemObjekt>();
        DavSystemObjekt              typMq;
        SystemObject                 soTyp;
        String                       soTypPid;
        Messquerschnitt              mq;

        // Hole alle MQs vom Typ TYP_MQ_ALLGEMEIN ab:
        for (SystemObject so : _davKonf.getType(TYP_MQ_ALLGEMEIN).getElements()) {
            soTyp    = so.getType();
            soTypPid = soTyp.getPidOrNameOrId();

            // Erstelle TypMessQuerschnittAllgemein:
            if (!mapTypMq.containsKey(soTypPid)) {
                typMq = new DavSystemObjekt(soTyp);
                mapTypMq.put(soTypPid, typMq);
            }

            typMq = mapTypMq.get(soTypPid);

            // Erstelle Eintrag im _mapTypMq:
            if (!_mapTypMq.containsKey(typMq)) {
                _mapTypMq.put(typMq, new ArrayList<Messquerschnitt>());
            }

            // Initialisiere Messquerschnitt:
            mq = mapMqZs.get(so.getPidOrNameOrId());

            if (mq == null) {
                mq = new Messquerschnitt((ConfigurationObject) so, null);
            }

            // Einfgen Messquerschnitt in _mapTypMq:
            _mapTypMq.get(typMq).add(mq);

            // Erstelle Eintrag im _mapMqTyp:
            _mapMqTyp.put(mq, typMq);
        }

        // -----------------------------------------------------------
        // Sortiere die MQ-Listen:
        for (DavSystemObjekt typ : _mapTypMq.keySet()) {
            Collections.sort(_mapTypMq.get(typ), DatenVew.COMPARATOR_SO_PID);
        }
    }

    /**
     * Pause bis dem Ende der Initialisierung
     * s.{@link #_istInitialisierungAbgeschlossen}.
     */
    private void pauseBisEndeDerInitialisierung() {
        while (!_istInitialisierungAbgeschlossen) {
            try {
                Thread.sleep(100);
            }
            catch (Exception e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 31.10.13
