/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.filter;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTM;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.navigation.Activator;
import de.inovat.buv.projektlib.speicher.GrundEinstellungen;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Menge der Filter die in der XML-Datei gespeichert werden.
 *
 * Diese Klasse ist als Singelton realisiert.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public final class AlleFilter {

    /**
     * Stellt fest, ob die Initialisierung der Instanz abgeschlossen ist (s.{@link #AlleFilter()}).
     *
     * WICHTIG:
     *
     * Die Methode {@link #getInstanz()} MUSS auf das Ende der Initialisierung warten.
     */
    private static boolean _istInitialisierungAbgeschlossen = false;

    /**
     * Einzige Instanz der Klasse fr den NavigationBaum:
     */
    private static AlleFilter _instanz = new AlleFilter();
    public static boolean     _klasseLaden;

    //~ FELDER ================================================================

    /**
     * Map enthlt alle definierte Filter.
     * Key: FilterName.
     */
    private Map<String, Filter> _filterMap;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    private AlleFilter() {

        // -------------------------------------------------------------------
        // WICHTIG
        // Die Initialisierung aller Filter luft langsam ===>
        // wird in JOB ausgelagert.
        // ===>
        // Die Methode {@link #getInstanz()} MUSS auf
        // das Ende der Initialisierung warten.
        Job job = new Job("Initialisiere alle Filter...") {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                ladeAlleFilter();

                //
                _istInitialisierungAbgeschlossen = true;

                return Status.OK_STATUS;
            }
        };

        job.schedule();
    }

    //~ METHODEN ==============================================================

    /**
     * Fge neues Filter in filterMap ein.
     *
     * @param filter - FilterObejekt.
     */
    public void addFilter(Filter filter) {
        _filterMap.put(filter.getFilterName(), filter);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof AlleFilter)) {
            return false;
        }

        AlleFilter other = (AlleFilter) obj;

        if (_filterMap == null) {
            if (other._filterMap != null) {
                return false;
            }
        } else if (!_filterMap.equals(other._filterMap)) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt das sortierte Array der Namen von definierten Filter zurck,
     * oder leeres StringArray, falls keine Filter definiert sind.
     *
     * @return Sortiertes Array der Namen von definierten Filter,
     * oder leeres StringArray, falls keine Filter definiert sind.
     */
    public String[] getFilterArray() {
        String[] ar = _filterMap.keySet().toArray(new String[_filterMap.keySet().size()]);

        Arrays.sort(ar);

        return ar;
    }

    /**
     * Gibt der Filter mit dem angegebenen Namen
     * oder null, falls solche nicht existiert.
     *
     * @param filterName - der Name des Filters.
     * @return Filter mit angegebenem Namen oder null.
     */
    public Filter getFilterObjekt(String filterName) {
        return _filterMap.get(filterName);
    }

    /**
     * Gibt einzige Instanz der Klasse fr Filter zurck.
     *
     * @return einzige Instanz der Klasse fr Filter.
     */
    public static AlleFilter getInstanz() {
        pauseBisEndeDerInitialisierung();

        return _instanz;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + ((_filterMap == null) ? 0 : _filterMap.hashCode());

        return result;
    }

    /**
     * Lade die XMLDatei ins filterMap.
     */
    @SuppressWarnings("unchecked")
    private void ladeAlleFilter() {
        _filterMap = new Hashtable<String, Filter>();

        // Ermittle die String-Representation des Maps aus dem Speicher:
        String mapString = GrundEinstellungen.ermittleParameterWertFuerAlleBenutzer(KonstantenGTM.ALLE_FILTER);

        if (mapString != null) {

            // Dekodiere den Parameter:
            try {
                ByteArrayInputStream input     = new ByteArrayInputStream(mapString.getBytes());
                XMLDecoder           xml       = new XMLDecoder(input);
                Object               mapObjekt = xml.readObject();

                xml.close();

                if (mapObjekt instanceof Map) {
                    try {
                        _filterMap = (Hashtable<String, Filter>) mapObjekt;
                    }
                    catch (Exception e) {
                        Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Filter knnen nicht geladen werden!", e));
                    }
                }
            }
            catch (Exception e) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Filter knnen nicht geladen werden!", e));
            }
        }
    }

    /**
     * Lsche ein Filter mit dem Namen filterName aus dem filterMap.
     *
     * @param filterName Name vom zu lschenden Filter.
     */
    public void loescheFilter(String filterName) {
        _filterMap.remove(filterName);
    }

    /**
     * Pause bis dem Ende der Initialisierung
     * s.{@link #_istInitialisierungAbgeschlossen}.
     */
    private static void pauseBisEndeDerInitialisierung() {
        while (!_istInitialisierungAbgeschlossen) {
            try {
                Thread.sleep(100);
            }
            catch (Exception e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }
    }

    /**
     * Speichere den Inhalt von _filterMap im Speicher.
     */
    public void speichereAlleFilter() {

        // Kodiere _filterMap ber XMLEncoder:
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        XMLEncoder            xml = new XMLEncoder(out);

        xml.writeObject(_filterMap);
        xml.close();

        boolean gespeichert = GrundEinstellungen.speichereParameterFuerAlleBenutzer(KonstantenGTM.ALLE_FILTER, out.toString(), "Speichern von Filtern");

        if (!gespeichert) {
            MessageDialog.openError(PlatformUI.getWorkbench().getDisplay().getActiveShell(), "Fehler beim Speichern", "Es ist ein Fehler beim Speichern des Filters aufgetretten." + "Details siehe unten im Meldungsfenster.");
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 05.11.13
