/*
 * @(#)DarstellungenVew.java   29.02.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gui.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.navigation.Activator;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet alle Darstellungen fr die Navigation.
 * Die Darstellungen sind als Extension definiert.
 * </p>
 * als Singleton realisiert.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class DarstellungenVew {

    /** Menge der fr DatenModell-nderungen registrierten Klassen */
    private static final Set<IDatenModell> mengeModellAenderungsListener = new HashSet<IDatenModell>();

    /** Einzige Instanz der Klasse. */
    private static DarstellungenVew instanz = new DarstellungenVew();

    //~ FELDER ================================================================

    /** Liste mit ID aller nach Namen sortierten Darstellungen. */
    private List<String> _listeDarstellungenID;

    /**
     * Map aller Darstellungen.
     * Key: ID der Darstellung.
     * Value: Darstellung.
     */
    private Map<String, IDarstellung> _mapDarstellungen;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse DarstellungenVew.
     */
    private DarstellungenVew() {

        //
        _mapDarstellungen     = new HashMap<String, IDarstellung>();
        _listeDarstellungenID = new ArrayList<String>();

        // Lade alle Darstellungen:
        ladeAlleDarstellungen();

        // Erstelle sortierte Liste aller Darstellungen-IDs:
        _listeDarstellungenID.addAll(_mapDarstellungen.keySet());
        Collections.sort(_listeDarstellungenID, new Comparator<String>() {
            @Override
            public int compare(String id1, String id2) {
                return _mapDarstellungen.get(id1).getName().compareTo(_mapDarstellungen.get(id2).getName());
            }
        });
    }

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IDatenModell}.
     *
     * @param klasse Klasse, die das Interface {@link IDatenModell} implementiert
     * und ber DatenModell-nderungen informiert werden will.
     */
    public static void addModellListener(IDatenModell klasse) {
        synchronized (mengeModellAenderungsListener) {
            mengeModellAenderungsListener.add(klasse);
        }
    }

    /**
     * Ruft bei nderungen im DatenModell die Methode {@link IDatenModell#modellGeaendert(IDarstellung))}()
     * fr jede als Listener registrierte Klasse auf.
     *
     * @param sicht Darstellung, deren DatenModell gendert wurde.
     */
    public static void benachrichtigeModellGeaendert(IDarstellung sicht) {
        synchronized (mengeModellAenderungsListener) {
            for (IDatenModell klasse : mengeModellAenderungsListener) {
                try {
                    klasse.modellGeaendert(sicht);
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        }
    }

    /**
     * Ermittle Namen aller Darstellungen als sortiertes Array.
     *
     * @return Array mit Namen aller Darstellungen.
     */
    public String[] ermittleAlleDarstellungen() {
        String[] ar = new String[_listeDarstellungenID.size()];
        int      i  = 0;

        for (String id : _listeDarstellungenID) {
            ar[i] = _mapDarstellungen.get(id).getName();
            i++;
        }

        return ar;
    }

    /**
     * Ermittle die Darstellung mit angegebenem Index.
     *
     * @param index Index der Darstellung.
     *
     * @return Darstellung mit angegebenem Index
     * oder null, falls nicht gefunden wird.
     */
    public IDarstellung ermittleDarstellung(int index) {
        try {
            return _mapDarstellungen.get(_listeDarstellungenID.get(index));
        }
        catch (Exception e) {
            return null;
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittelt einzige Instanz der Klasse DarstellungenVew.
     *
     * @return Einzige Instanz der Klasse DarstellungenVew.
     */
    public static DarstellungenVew getInstanz() {
        return instanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Lade alle Darstellungen, die als Extension definiert sind.
     */
    private void ladeAlleDarstellungen() {
        IExtensionRegistry reg = Platform.getExtensionRegistry();

        if (reg != null) {
            String       id;
            IDarstellung klasse;

            for (IConfigurationElement el : reg.getConfigurationElementsFor(Activator.PLUGIN_ID, "NavigationDarstellung")) {
                try {

                    // Lese Extension:
                    id     = el.getAttribute("id");
                    klasse = (IDarstellung) el.createExecutableExtension("class");

                    // Prfe Extension:
                    if (klasse.getName() == null) {
                        throw new Exception(String.format("Fehler im Extension <%s>: Methode getName() liefert null", id));
                    }

                    if (klasse.getContentProvider() == null) {
                        throw new Exception(String.format("Fehler im Extension <%s>: Methode getContentProvider() liefert null", id));
                    }

                    if (klasse.getLabelProvider() == null) {
                        throw new Exception(String.format("Fehler im Extension <%s>: Methode getLabelProvider() liefert null", id));
                    }

                    /* Die Prfung wird ausgelagert, um am Anfang die Zeit sparen:
                    if (klasse.getInput() == null) {
                        throw new Exception(String.format("_Fehler im Extension <%s>: Methode klasse.getInput() liefert null", id));
                    }

                    if (!(klasse.getInput() instanceof List)) {
                        throw new Exception(String.format("_Fehler im Extension <%s>: Methode klasse.getInput() muss eine Liste liefern", id));
                    }
                    */

                    // Einfgen:
                    if (_mapDarstellungen.put(id, klasse) != null) {
                        Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format("Darstellung mit ID <%s> wurde berschrieben", id)));
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        }
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf {@link IDatenModell}.
     *
     * @param klasse Klasse, die sich zuvor mittels {@link #addModellListener(IDatenModell)} registriert hat.
     */
    public static void removeModellListener(IDatenModell klasse) {
        synchronized (mengeModellAenderungsListener) {
            mengeModellAenderungsListener.remove(klasse);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 12.03.13
