/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.bundesland;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.dav.DavDatenVew;
import de.inovat.buv.projektlib.konstanten.AufrufParameter;
import de.inovat.buv.projektlib.konstanten.KonstantenGUIResource;
import de.inovat.buv.projektlib.konstanten.Zustaende;
import de.inovat.buv.projektlib.konstanten.Zustaende.BundesLaender;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.FILESEPARATOR;
import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.io.File;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Resourcen fr das im Aufrufparameter bergebene Bundesland.
 *
 * Alls Singleton realisiert.
 *
 *
 * Einstellungen fr das neue Bundesland:
 *
 *      -       Bundesland in Enum ..konstanten.Zustaende.BundesLaender einfgen.
 *
 *      -       ResourceOrdner mit dem Namen <<Abkrzung des Landes>> in
 *              ..\[ProjektOrdner]\ProjektResource erstellen.
 *
 *      -       In ResourceOrdner folgende Dateien einfgen:
 *                      OberflaechenFunktionen.csv,
 *                      ...
 *
 *      -       Die Aktionen in plugin.xml registrieren.
 *
 *      -       Datei 'Logo<<Abkrzung des Landes>>.png' mit dem Logo in Ordner
 *              ..icons\logos hinzufgen.
 *
 *      ...
 *
 *
 * TIPP: alle Aufrufe ber BundeslandVew.getInstanz() verfolgen.
 *
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 38 $ / $Date: 2013-10-08 15:30:34 +0200 (Di, 08 Okt 2013) $ / ($Author: LG $)
 */
public class BundeslandVew {
    private static final String ORDNER_RESOURCE = "ProjektResource";

    /** Einzige Instanz der Klasse. */
    private static BundeslandVew _instanzBundeslandVew = new BundeslandVew();

    //~ FELDER ================================================================

    /**
     * FehlerMeldung, falls der ProjektResource-Ordner nicht gefunden wird,
     * sonst - leeres String.
     */
    private String _fehlerMeldungFuerOrdner = "";

    /** Stellt fest, ob das Bundesland vom Aufrufparameter geladen wurde. */
    private boolean _bundeslandGeladen = false;

    /** Das im Aufrufparameter bergebene Bundesland oder unbekannt (DefaultOrdner AA). */
    private Zustaende.BundesLaender _bundesland;

    /** Ordner, wo alle BundeslandResource gespeichert sind oder leeres String, falls nicht definiert wird. */
    private String _ordnerBundeslandResource;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse BundeslandVew.
     */
    private BundeslandVew() {
        initialisiereBundesland();
        initialisiereOrdnerBundeslandResource();
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittelt die Datei mit angegebenem Namen, die im OrdnerBundeslandResource liegt.
     *
     * @param datei Name der Datei.
     * Soll als Konstante definiert werden.
     *
     * @return Datei mit angegebenem Namen, die im OrdnerBundeslandResource liegt.
     */
    public String ermittleDateiImOrdnerBundeslandResource(String datei) {
        return getOrdnerBundeslandResource() + FILESEPARATOR + datei;
    }

    /**
     * Ermittle das Logo des Bundeslandes.
     *
     * @return Logo des Bundeslande aus dem Aufrufparameter
     * oder Standard-Logo des Bundeslandes.
     */
    public Image ermittleLogoBundesland() {
        if (KonstantenGUIResource.ICON_LOGO_BUNDESLAND_AUS_PARAMETER != null) {
            return KonstantenGUIResource.ICON_LOGO_BUNDESLAND_AUS_PARAMETER;
        }

        return KonstantenGUIResource.MAP_ICON_LOGO_BUNDESLAND.get(_bundesland.getAbkuerzung());
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'bundesland'.
     *
     *     @return Wert der Eigenschaft 'bundesland'.
     *
     *     @see #_bundesland
     */
    public Zustaende.BundesLaender getBundesland() {
        return _bundesland;
    }

    /**
     * Gibt die Instanz der BundeslandVew zurck. <p/>
     * Die BundeslandVew ist als Singleton realisiert und kann nicht direkt instanziert werden.
     * Der Zugriff auf die Methoden erfolgt ausschlielich ber die Instanz.
     *
     * @return Instanz der Klasse BundeslandVew.
     */
    public static BundeslandVew getInstanz() {
        return _instanzBundeslandVew;
    }

    /**
     * Getter fr Eigenschaft 'ordnerBundeslandResource'.
     *
     * @return Wert der Eigenschaft 'ordnerBundeslandResource'.
     *
     * @see #_ordnerBundeslandResource
     */
    public String getOrdnerBundeslandResource() {
        if (!_fehlerMeldungFuerOrdner.isEmpty()) {
            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                    MessageDialog.openError(shell, "Fehler", _fehlerMeldungFuerOrdner);
                }
            });
        }

        return _ordnerBundeslandResource;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere das Bundesland
     */
    private void initialisiereBundesland() {

        // Hole das KommandoZeilenParameter fr das Bundesland:
        String param = DavDatenVew.getInstanz().getAufrufParameterAlsString(AufrufParameter.BUNDESLAND);

        if (param != null) {
            _bundesland = Zustaende.BundesLaender.ermittleBundesland(param);
        }

        // FehlerMeldung:
        if ((_bundesland == null) || (_bundesland == BundesLaender.UNBEKANNT)) {
            _bundesland = BundesLaender.UNBEKANNT;
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, String.format("%s%s%s", "Das Bundesland ist im Kommandozeilenparameter <", AufrufParameter.BUNDESLAND, "> nicht definiert!")));
        }

        _bundeslandGeladen = true;
        System.out.println("Bundesland: " + _bundesland.getLand());
    }

    /**
     * Initialisiere den BundeslandResource-Ordner.
     */
    private void initialisiereOrdnerBundeslandResource() {

        // D:\Entwicklung\PROJEKT_NAME\ORDNER_RESOURCE\_bundesland.getAbkuerzung()
        String resOrd = ORDNER_RESOURCE + FILESEPARATOR + _bundesland.getAbkuerzung();
        String par    = AufrufParameter.RESOURCE_ORDNER;

        // Zuerst wird im ProjektRootOrdner gesucht.
        // Falls nicht gefunden --> im Ordner aus dem Aufrufparameter.
        String ordnerPrjRoot;
        String ordnerPar = "";

        // Produkt (ProjektRootOrdner):
        try {
            ordnerPrjRoot = Platform.getInstallLocation().getURL().getPath();
        }
        catch (Exception e) {
            ordnerPrjRoot = "";
        }

        ordnerPrjRoot = String.format("%s%s", ordnerPrjRoot, resOrd);

        if (new File(ordnerPrjRoot).exists()) {
            _ordnerBundeslandResource = ordnerPrjRoot;
        }

        // Ordner aus dem Aufrufparameter (z.B in Eclipse whren der EntwicklungsPhase):
        else {

            /*
        try {
            URL url = de.inovat.buv.projektlib.bundesland.BundeslandVew.class.getResource(getClass().getSimpleName() + ".class");

            ordner = FileLocator.toFileURL(url).getPath();
            ordner = ordner.substring(0, ordner.indexOf("/swe/"));
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            ordner = "";
        }
        */

            // Hole das KommandoZeilenParameter fr den resourceOrdner:
            String ord = DavDatenVew.getInstanz().getAufrufParameterAlsString(par);

            if (ord != null) {
                ordnerPar = String.format("%s%s%s", ord, FILESEPARATOR, resOrd);

                if (new File(ordnerPar).exists()) {
                    _ordnerBundeslandResource = ordnerPar;
                }
            }
        }

        // FehlerMeldung:
        if (_ordnerBundeslandResource == null) {
            _ordnerBundeslandResource = "";
            _fehlerMeldungFuerOrdner  = String.format("%s%s%s%s%s%s%s",                                                    //
                                                      "Der ProjektResource-Ordner fr das Projekt wurde nicht gefunden!",  //
                                                      NEWLINE, NEWLINE,                                                    //
                                                      "Er wurde zuerst in <", ordnerPrjRoot, "> gesucht.", NEWLINE);

            if (ordnerPar.isEmpty()) {
                _fehlerMeldungFuerOrdner = String.format("%s%s%s%s", _fehlerMeldungFuerOrdner,                             //
                                                         "Danach wrde der ber den Aufrufparameter <", par, "> angegebene Ordner geprft.");
            } else {
                _fehlerMeldungFuerOrdner = String.format("%s%s%s%s%s%s", _fehlerMeldungFuerOrdner,                         //
                                                         "Danach wurde der ber den Aufrufparameter <", par, "> angegebene Ordner <", ordnerPar, "> geprft.");
            }

            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, _fehlerMeldungFuerOrdner));
        }

        //
        System.out.println("BundeslandResource-Ordner: " + _ordnerBundeslandResource);
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'bundeslandGeladen'.
     *
     *     @return Wert der Eigenschaft 'bundeslandGeladen'.
     *
     *     @see #_bundeslandGeladen
     */
    public boolean isBundeslandGeladen() {
        return _bundeslandGeladen;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.10.13
