/*
 * @(#)DavDatenVew.java   26.05.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.dav;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;

import de.inovat.buv.projektlib.rwservice.RWService;

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt eine Zwischenschicht zum Rahmenwerk-Service 'Rahmenwerk'
 * {@link RWService#getRw()} dar.
 * </p>
 * Klasse ermittelt aktuelle Datenverteilerverbindung, Aufrufparameter,
 * verwaltet DaV-Verbindung Listener.
 *
 *
 * Klasse stellt die DAV-Daten zu Verfgung.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 40 $ / $Date: 2013-10-09 09:54:35 +0200 (Mi, 09 Okt 2013) $ / ($Author: LG $)
 */
public class DavDatenVew {

    /** Einzige Instanz der Klasse. */
    private static DavDatenVew _instanz = new DavDatenVew();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse DavDatenVew.
     */
    private DavDatenVew() {}

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IDavVerbindung}.
     *
     * @param klasse Klasse, die das Interface {@link IDavVerbindung}
     * implementiert und ber nderungen der DaV-Verbindung informiert werden will.
     */
    public void addDavVerbindungListener(IDavVerbindung klasse) {
        RWService.getService().getRw().addDavVerbindungsListener(klasse);
    }

    /**
     * Die Methode bestimmeStatusText bestimmt zu den Daten des Datenverteilers
     * den Status.
     *
     * @param resultDataState
     *            Status des Data Objektes vom Datenverteiler.
     * @return Status als String.
     */
    public String bestimmeStatusText(DataState resultDataState) {
        String statusText;

        if (resultDataState == DataState.NO_DATA) {
            statusText = "keine Daten";
        } else if (resultDataState == DataState.NO_SOURCE) {
            statusText = "keine Daten (keine Quelle)";
        } else if (resultDataState == DataState.NO_RIGHTS) {
            statusText = "keine Daten (keine Rechte)";
        } else if (resultDataState == DataState.POSSIBLE_GAP) {
            statusText = "keine Daten (potentielle Datenlcke)";
        } else if (resultDataState == DataState.END_OF_ARCHIVE) {
            statusText = "Ende des Archivanfragezeitraums";
        } else if (resultDataState == DataState.DELETED_BLOCK) {
            statusText = "Gelschter Bereich";
        } else if (resultDataState == DataState.UNAVAILABLE_BLOCK) {
            statusText = "Ausgelagerter Bereich";
        } else if (resultDataState == DataState.INVALID_SUBSCRIPTION) {
            statusText = "keine Daten (fehlerhafte Anmeldung)";
        } else {
            statusText = "keine Daten (Undefinierte Objektkodierung)";
        }

        return statusText;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittle den Wert des Aufrufparameters.
     *
     * @param parameter Aufrufparameter.
     *
     * @return Wert des Aufrufparameters oder null.
     */
    public Argument getAufrufParameter(String parameter) {
        try {
            return RWService.getService().getRw().getStartParameter().fetchArgument(parameter);
        }
        catch (Exception e) {
            return null;
        }
    }

    /**
     * Ermittle den logischen Wert des Aufrufparameters.
     *
     * @param parameter Aufrufparameter.
     * @param defaultWert Default-Wert, falls der Aufrufparameter nicht ermittelt werden kann.
     *
     * @return Logischer Wert des Aufrufparameters oder defaultWert,
     * falls der Aufrufparameter nicht ermittelt werden kann.
     */
    public boolean getAufrufParameterAlsBoolean(String parameter, boolean defaultWert) {
        try {
            return RWService.getService().getRw().getStartParameter().fetchArgument(parameter).booleanValue();
        }
        catch (Exception e) {
            return defaultWert;
        }
    }

    /**
     * Ermittle den Wert des Aufrufparameters als String.
     *
     * @param parameter Aufrufparameter.
     *
     * @return Wert des Aufrufparameters als String oder null.
     */
    public String getAufrufParameterAlsString(String parameter) {
        try {
            return RWService.getService().getRw().getStartParameter().fetchArgument(parameter).asNonEmptyString().trim();
        }
        catch (Exception e) {
            return null;
        }
    }

    /**
     * Ermittle die aktuelle Datenverteilerverbindung.
     *
     * @return Aktuelle Datenverteilerverbindung.
     */
    public ClientDavInterface getDav() {
        return RWService.getService().getRw().getDavVerbindung();
    }

    /**
     * Ermittle einzige Instanz der Klasse.
     *
     * @return Einzige Instanz der Klasse.
     */
    public static DavDatenVew getInstanz() {
        return _instanz;
    }

    /**
     * Ermittle das definierte Oberflchenobjekt.
     *
     * @return Das definierte Oberflchenobjekt oder null.
     */
    public SystemObject getOberflaechenObject() {
        return RWService.getService().getRw().getOberflaechenObject();
    }

    //~ METHODEN ==============================================================

    /**
     * Lscht die Registrierung einer Klasse als Listener auf
     * {@link IDavVerbindung}.
     *
     * @param klasse Klasse, die sich zuvor mittels
     * {@link #addDavVerbindungListener(IDavVerbindung)} registriert hat.
     */
    public void removeDavVerbindungListener(IDavVerbindung klasse) {
        RWService.getService().getRw().removeDavVerbindungsListener(klasse);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 09.10.13
