/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.rechtevew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.berechtigung.Berechtigungen;
import de.bsvrz.buv.rw.basislib.berechtigung.BerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.IBerechtigungListener;

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.konstanten.AufrufParameter;
import de.inovat.buv.projektlib.rwservice.RWService;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt eine Zwischenschicht zum Rahmenwerk-Service 'Berechtigungen'
 * {@link RWService#getBerechtigungen()} dar.
 *
 * <h2> Berechtigungskonzept:</h2>
 * </p>
 * Definiert die Rechte von Benutzern (anhand von der Zuordnung eines Benutzers zu einer
 * Berechtigungsklasse) auf Funktionen (konkrete Aktionen oder andere Eigenschaften).
 * </p>
 * Es werden folgende Aufrufparameter bercksichtigt:
 * <ul>
 * <li>
 * {@link AufrufParameter#BERECHTIGUNGS_KONZEPT}
 * </p>
 * Default Wert ist ja. Beim ausgeschalteten Berechtigungskonzept sind alle Funktionen erlaubt.
 * </li>
 * <li>
 * AufrufParameter#OBERFLAECHEN_OBJEKT
 * </p>
 * Von diesem DaV-Objekt werden die Rechte geladen.
 * Falls der Aufrufparameter nicht gesetzt ist, wird das Standard-Objekt geladen.
 * </li>
 * </ul>
 * <h3>Berechtigungsfunktionen:</h3>
 * sind in Plugins als Extensions de.bsvrz.buv.rw.basislib.funktionmitberechtigung regisrtiert.
 * <h3>Struktur der Daten im DaV:</h3>
 * <ul>
 * </p>
 * Oberflche (typ.oberflche) - OberflchenBerechtigung (atg.oberflchenBerechtigung)
 * </p>
 * Array OberflchenBerechtigung:
 * <ul>
 * </p>
 *      Liste von Berechtigungsklassen (knnen '*' enthalten);
 * </p>
 *      Liste von Funktionen (knnen '*' enthalten);
 * </p>
 *      Berechtigung der aufgelisteten Berechtigungsklassen auf die aufgelisteten Funktionen (Freigabe/Sperrung);
 * </p>
 *      DaV-Felder ObjektMenge, Ausnahmen und Ebene werden im Berechtigungskonzept nicht bercksichtigt.
 * </ul>
 * </p>
 * </ul>
 * </p>
 * </p>
 * Listener:
 * Klasse meldet sich als Zuhrer der Rechten-nderungen und verwaltet den Rechten-Listener
 * fr andere Zuhrer, so dass die eine Benachrichtigung nur dann bekommen, wenn der Recht des
 * angemeldeten Benutzers auf die zu berwachende Funktion gendert wurde.
 * So wird die mehrmalige Benachrichtigung der Listener verhindert.
 * (RW-Berechtigungen: Bei der nderung des Rechtes auf einer Funktion werden alle Listener
 * unabhngig davon, ob die diese Funktion berwachen, benachrichtigt)
 * </p>
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 102 $ / $Date: 2013-12-04 12:08:03 +0100 (Mi, 04 Dez 2013) $ / ($Author: LG $)
 */
public final class RechteVew implements IBerechtigungListener {

    /** Instanz der Berechtigungen-Klasse aus dem RWService. */
    private static Berechtigungen berechtigungen = RWService.getService().getBerechtigungen();

    /** Einzige Instanz der Klasse. */
    private static RechteVew instanz = new RechteVew();

    /**
     * Menge der auf Rechtenderungen registrierten Listener.
     * </p>
     * Key: ID der Berechtigunsfunktion.
     * </p>
     * Value: Menge, die fr jede registrierte Klasse den Rechten-Wert
     * des angemeldeten Benutzer auf die Funktion enthlt.
     * Bei der nderung der Rechte wird dieser Wert mit neuem
     * verglichen und die Klasse bekommt die Benachrichtigung, nur falls
     * der Wert gendert wurde.
     */
    private static final Map<String, Map<IRechteAenderung, Boolean>> mapRechteListener = new HashMap<String, Map<IRechteAenderung, Boolean>>();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse RechteVew.
     */
    private RechteVew() {

        // Registriere die Klasse als Zuhrer der Rechten-nderungen fr alle Funktionen:
        berechtigungen.addOberflaechenFunktionsListener(this);
    }

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IRechteAenderung}.
     *
     * @param klasse Klasse, die das Interface {@link IRechteAenderung} implementiert und
     * ber alle Rechtenderungen informiert werden will.
     */
    public static void addRechteListener(IRechteAenderung klasse) {
        berechtigungen.addOberflaechenFunktionsListener(klasse);
    }

    /**
     * Registriert Klasse als Listener auf {@link IRechteAenderung}.
     *
     * @param klasse Klasse, die das Interface {@link IRechteAenderung} implementiert und
     * ber Rechtenderungen informiert werden will.
     *
     * @param listeFunktionenId Die Liste der von der Klasse zu berwachten Funktionen.
     */
    public static void addRechteListener(IRechteAenderung klasse, List<String> listeFunktionenId) {
        FunktionMitBerechtigung funk;

        for (String funkId : listeFunktionenId) {
            funk = berechtigungen.getFunktion(funkId);

            if (funk != null) {
                synchronized (mapRechteListener) {
                    Map<IRechteAenderung, Boolean> mapRechte = mapRechteListener.get(funkId);

                    if (mapRechte == null) {

                        // Erstelle die Menge der Klassen mit Rechten-Werten fr die Funktion:
                        mapRechte = new HashMap<IRechteAenderung, Boolean>();
                        mapRechteListener.put(funkId, mapRechte);
                    }

                    mapRechte.put(klasse, null);
                }
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Fehler beim Regestrieren der Klasse <%s> als Listener auf Rechtenderungen: undefinierte Funktion <%s>", klasse.getClass().getName(), funkId)));
            }
        }
    }

    /**
     * Registriert Klasse als Listener auf {@link IRechteAenderung}.
     *
     * @param klasse Klasse, die das Interface {@link IRechteAenderung} implementiert und
     * ber Rechtenderungen informiert werden will.
     *
     * @param funktionId Die von der Klasse zu berwachte Funktion.
     */
    public static void addRechteListener(IRechteAenderung klasse, String funktionId) {
        List<String> listeFunktionenId = new ArrayList<String>();

        listeFunktionenId.add(funktionId);
        addRechteListener(klasse, listeFunktionenId);
    }

    @Override
    public void freigabe(BerechtigungEreignis e) {
        rechteGeaendert(e);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittle einzige Instanz der Klasse.
     *
     * @return Einzige Instanz der Klasse.
     */
    public static RechteVew getInstanz() {
        return instanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Stellt fest, ob die angegebene Funktion fr den angemeldeten Benutzer erlaubt ist.
     *
     * @param funktionId ID der zu prfenden Funktion.
     *
     * @return true, falls die angegebene Funktion fr den angemeldeten Benutzer erlaubt ist,
     * sonst - false.
     */
    public boolean istFunktionFuerAngemeldetenBenutzerErlaubt(String funktionId) {
        FunktionMitBerechtigung funk = berechtigungen.getFunktion(funktionId);

        if (funk != null) {
            return berechtigungen.hasBerechtigung(funk);
        } else {
            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                "Fehler beim Ermitteln der Berechtigung: undefinierte Funktion <%s>", funktionId)));

            return false;
        }
    }

    /**
     * Stellt fest, ob die angegebene Funktion fr den angegebenen Benutzer erlaubt ist.
     *
     * @param benutzerPid PID des Benutzers.
     * @param funktionId ID der zu prfenden Funktion.
     *
     * @return true, falls die angegebene Funktion fr den Benutzer erlaubt ist,
     * sonst - false.
     */
    public boolean istFunktionFuerBenutzerErlaubt(String benutzerPid, String funktionId) {
        FunktionMitBerechtigung funk = berechtigungen.getFunktion(funktionId);

        if (funk != null) {
            return berechtigungen.hasBerechtigung(benutzerPid, funk);
        } else {
            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                "Fehler beim Ermitteln der Berechtigung: undefinierte Funktion <%s>", funktionId)));

            return false;
        }
    }

    /**
     * Reagiere auf die nderungen der Rechte.
     *
     * @param e BerechtigungEreignis.
     */
    private void rechteGeaendert(BerechtigungEreignis e) {
        synchronized (mapRechteListener) {
            Map<IRechteAenderung, Boolean> mapRechte;
            Boolean                        rechtAlt;
            Boolean                        rechtNeu;

            // Betrachte die Funktion, fr die die nderung gemeldet wird:
            for (FunktionMitBerechtigung funk : e.getFunktionen()) {
                mapRechte = mapRechteListener.get(funk.getId());

                // Falls es Zuhrer fr diese Funktion gibt:
                if (mapRechte != null) {

                    // Ermittle neuen Wert:
                    rechtNeu = istFunktionFuerAngemeldetenBenutzerErlaubt(funk.getId());

                    // Betrachte alle Zuhrer fr diese Funktion:
                    for (IRechteAenderung klasse : mapRechte.keySet()) {

                        // Setze neuen Wert
                        rechtAlt = mapRechte.put(klasse, rechtNeu);

                        // Falls der Wert gendert wurde, benachrichtige den Zuhrer::
                        if (!rechtNeu.equals(rechtAlt)) {

                            // Hier werden die Rechte fr den angemeldeten Benutzer ermittelt und die BerechtigungsKlasse ist unrelevant
                            // (fr neue Berechtigung kann der Benutzer zur mehreren BerechtigungsKlassen gehren)
                            // ===> berechtigungsKlasse = null
                            BerechtigungEreignis ereignis = new BerechtigungEreignis(this, rechtNeu, null, Collections.singleton(funk));

                            if (rechtNeu) {
                                klasse.freigabe(ereignis);
                            } else {
                                klasse.sperrung(ereignis);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf {@link IRechteAenderung}.
     *
     * @param klasse Klasse, die sich zuvor mittels addRechteListener(...) registriert hat.
     */
    public static void removeRechteListener(IRechteAenderung klasse) {
        synchronized (mapRechteListener) {
            Set<String>                    funktionenLoeschen = new HashSet<String>();
            Map<IRechteAenderung, Boolean> mapRechte;

            for (String funkId : mapRechteListener.keySet()) {

                // Lsche die Klasse aus der Menge der Klassen mit Rechten-Werten fr die Funktion:
                mapRechte = mapRechteListener.get(funkId);
                mapRechte.remove(klasse);

                // Falls die Menge der Klassen leer ist, merke die Funktion fr das Lschen aus der Menge der Listener:
                if (mapRechte.isEmpty()) {
                    funktionenLoeschen.add(funkId);
                }
            }

            // Lsche die Funktionen, die von niemandem mehr berwacht werden, aus der Menge der Listener:
            for (String funkId : funktionenLoeschen) {
                mapRechteListener.remove(funkId);
            }
        }
    }

    @Override
    public void sperrung(BerechtigungEreignis e) {
        rechteGeaendert(e);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 04.12.13
